package org.eso.ohs.core.dbb.client;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.Point;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.lang.reflect.InvocationTargetException;
import java.net.MalformedURLException;
import java.rmi.NotBoundException;
import java.rmi.RemoteException;
import java.sql.SQLException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.Vector;

import javax.swing.Action;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.ListSelectionModel;
import javax.swing.SwingUtilities;
import javax.swing.event.EventListenerList;
import javax.swing.event.ListSelectionListener;

import org.apache.log4j.Logger;
import org.eso.ohs.core.dbb.server.DbbQuery;
import org.eso.ohs.core.dbb.server.DbbSqlEngine;
import org.eso.ohs.core.dbb.sql.DbbCountStarted;
import org.eso.ohs.core.dbb.sql.DbbInvalidSqlOperatorType;
import org.eso.ohs.core.dbb.sql.DbbQueryReset;
import org.eso.ohs.core.dbb.sql.DbbQueryStarted;
import org.eso.ohs.core.dbb.sql.DbbQueryStopped;
import org.eso.ohs.core.dbb.sql.DbbQueryTerminated;
import org.eso.ohs.core.dbb.sql.DbbQueryTerminatedListener;
import org.eso.ohs.core.dbb.sql.DbbSqlChunk;
import org.eso.ohs.core.dbb.sql.DbbSqlListener;
import org.eso.ohs.core.dbb.sql.DbbSqlOperator;
import org.eso.ohs.core.dbb.sql.DbbSqlQueryChunks;
import org.eso.ohs.core.dbb.sql.DbbSqlQueryResult;
import org.eso.ohs.core.dbb.sql.DbbSqlTable;
import org.eso.ohs.core.utilities.MsgManager;

/**summaryButtonssummaryButtons
 * This is the main GUI of a database browser. It is the client part of the
 * database browser: it collects all the chunk information coming from the
 * DbbWidgets, talks to the server (DbbSqlEngine) sending all the sql chunks
 * and getting the query results. It delegates the summary table (DbbTable
 * aggregate) to display the results.
 * <p>
 * @version $Revision: 1.21 $
 * @author Carlo Boarotto, ESO 19-Jul-1999
 * @see org.eso.ohs.core.dbb.server.DbbSqlEngine
 */
public class DbbView
	extends JPanel
	implements DbbSqlListener, DbbTableListener {


	/**
	 * Field cvsID_
	 */
	public static final String cvsID_ =
		"$Id: DbbView.java,v 1.21 2005/11/16 00:50:33 tcanavan Exp $";

	/** Thread object to run the queries. */
	private DBIOThread ioThread_;

	/**
	 * Field stdlog_
	 */
	private static Logger stdlog_ = Logger.getLogger(DbbView.class);
	/**
		 * Field stdlog_
		 */
		private static DbbVerifier verifier_ = new DbbVerifier();

	/**
	 * Boolean flag to indicate if the column classes have been already
	 * retrieved from the column selction chunks
	 */
	protected boolean columnClassesDefined_ = false;

	/**
	 * DbbPanel for selecting which columns will be displayed
	 * columnsPanel_ is a subset of columnsPanelFull_
	 */
	protected DbbPanel columnsPanel_ = null;
	/**
	 * columnsPanelFull hold the complete set of the rows
	 * widget read by the Dbb at the application startup
	 */
	private DbbPanel columnsPanelFull_ = null;

	/**
	 * DbbPanel for selecting which rows will be retrieved; i.e., the
	 * widgets that will generate the "WHERE" clauses.
	 * wherePanel_ is a subset of rowsPanelFull_
	 * TODO we should rename wherePanel_ as rowsPanel_
	 */
	protected DbbPanel wherePanel_ = null;
	/**
	 * rowsPanelFull hold the complete set of the rows
	 * widget read by the Dbb at the application startup
	 */
	private DbbPanel rowsPanelFull_ = null;
	/**
		* DbbPanel for selecting which rows will be retrieved; i.e., the
		* widgets that will generate the "WHERE" clauses.
		*/
	protected DbbPanel whereUnionPanel_ = null;

	/**
	 * DbbPanel for the widgets that will generate the "ORDER BY" clauses
	 */
	protected DbbPanel sortPanel_ = null;

	/**
	 * sortingPanelFull hold the complete set of the sorting
	 * widget read by the Dbb at the application startup
	 */
	private DbbPanel sortPanelFull_ = null;
	/**
	 * DbbPanel for the widgets that will generate the "GROUP BY" clauses
	 */
	protected DbbPanel groupPanel_ = null;

	/** Grid to display the query result set summary. */
	protected DbbTable summaryTable_ = null;

	/** Panel with summary command buttons. */
	protected DbbButtonPanel summaryButtons_ = null;

	/** Text field to give feedback to the user. */
	protected JTextField message_ = null;

	/** Vector of default chunks */
	protected Vector defaultChunks_ = new Vector();

	/** Reference to the detail view panel. */
	protected JPanel detail_ = null;

	/** TBD: optional menubar. */
	protected JMenuBar menubar_ = null;

	/** TBD: optional menubar panel. */
	protected MenuBarPanel menubarPanel_ = null;

	/** SQL engine associated to this view. */
	protected DbbSqlEngine sqlEngine_;

	/** Container located at the top of the table. */
	protected DbbViewContainer northPanel_;

	/** Container located at the bottom of the table. */
	protected DbbViewContainer southPanel_;

	/** Container located at the right of the table. */
	protected DbbViewContainer eastPanel_;

	/** Container located at the left of the table. */
	protected DbbViewContainer westPanel_;
	
	
	/* Button to display the dbbtable in text format*/
	
	protected JButton displayTextButton_;
	
	/*Display text report title */
	
	protected String displayTextReportTitle_ = "";
	
	
	protected String displayTextHeadingName_ = "";
	
	

	/** JFrame for the notification mechanism */
	private JFrame frame_;

	/** JPanel for the notification mechanism */
	protected JPanel glassPane_;

	/** Boolean flag to indicate that the query is in progrtess*/
	private boolean inProgress_ = false;

	/**
	 * Field mouseListener_
	 */
	private PanelMouseListener mouseListener_ = null;

	/**
	 * Field counter_
	 */
	private int counter_ = 0;
	
	private boolean isStartQueryOnEnter = false;
	
	private ArrayList mandatoryColumnWidgets = new ArrayList();
	
	private ArrayList mandatoryRowWidgets = new ArrayList();

	private ArrayList mandatorySortWidgets = new ArrayList();
	//----------------------------------------
	// Constructor
	//----------------------------------------

	/**
	 * Build a database browser view
	 *
	 * @param sqlEngine Reference to the SQL engine which does all the work
	 */
	public DbbView(DbbSqlEngine sqlEngine) {

		/* Call the JPanel constructor. */
		super(true);
		setOpaque(true);

		/* Initialize internal data structures. */
		sqlEngine_ = sqlEngine;
		stdlog_.debug("Before build view");
		buildView();
	}

	/**
	 * When the dbb gets added to a JFrame it adds a listener to the
	 * GlassPane. In order for the frame that the dbb is added to to be
	 * disposed properly this mouse listener must be removed. Please call
	 * this method when disposing of a JFrame with a dbb inside.
	 */

	public void removeGlassPaneMouseListener() {
		glassPane_.removeMouseListener(mouseListener_);
	}

	//----------------------------------------
	// Protected methods
	//----------------------------------------

	/**
	 * Method buildGrid
	 */
	protected void buildGrid() {
		/* Create the table at the center */
		summaryTable_ = new DbbTable();
		add(summaryTable_.getScrollPanel(), "Center");
	}

	/**
	 * Create the view layout adding the default panels
	 */
	protected void buildView() {

		/* Layout manager. */
		setLayout(new BorderLayout());

		buildGrid();

		/* Create a panel at the north */
		JPanel north_container = new JPanel(true);
		north_container.setOpaque(true);
		north_container.setLayout(new BorderLayout());

		northPanel_ = new DbbViewContainer();

		// Add the menubar panel
		menubarPanel_ = new MenuBarPanel();
		north_container.add(menubarPanel_, "North");

		north_container.add(northPanel_, "Center");
		add(north_container, "North");

		// /* Create a panel at the north */
		// northPanel_ = new DbbViewContainer();
		// add(northPanel_, "North");

		/* Create a panel at the south */
		JPanel south_container = new JPanel(true);
		south_container.setOpaque(true);
		south_container.setLayout(new BorderLayout());

		summaryButtons_ = new DbbButtonPanel();
		summaryButtons_.addListener(this);
		summaryButtons_.addTableListener(this);
		
		
		south_container.add(summaryButtons_, "North");

		message_ = new JTextField();
		message_.setEditable(false);

		south_container.add(message_, "South");
		southPanel_ = new DbbViewContainer();

		south_container.add(southPanel_, "Center");
		add(south_container, "South");

		/* Create a panel at the north */
		eastPanel_ = new DbbViewContainer();
		add(eastPanel_, "East");

		/* Create a panel at the north */
		westPanel_ = new DbbViewContainer();
		add(westPanel_, "West");
	}

	/**
	 * Add a view panel and display it at a specific position
	 *
	 * @param panel     View panel to be displayed
	 * @param position  Relative position in the view (NSEW)
	 * @param x         X position in the container grid
	 * @param y         Y position in the container grid
	 */
	protected void addJPanel(JPanel panel, String position, int x, int y) {

		/* Wrong vales: do nothing */
		if (position == null || panel == null)
			return;

		DbbViewContainer destination;

		String str = position.toUpperCase().trim();
		char p = str.charAt(0);

		/* Check the position in the view */
		switch (p) {
			case 'N' :
				destination = northPanel_;
				break;
			case 'S' :
				destination = southPanel_;
				break;
			case 'E' :
				destination = eastPanel_;
				break;
			case 'W' :
				destination = westPanel_;
				break;
			default :
				return;
		}

		/* If (x,y) parameters are given, add the panel in the (x,y)
		 * position within the container grid
		 */
		if (x >= 0 && y >= 0) {
			destination.gbc.gridwidth = 1;
			destination.gbc.gridheight = 1;
			destination.gbc.gridx = x;
			destination.gbc.gridy = y;
		} else {
			/* pack top/bottom */
			destination.gbc.gridwidth = GridBagConstraints.REMAINDER;
			//destination.gbc.gridwidth = 1;
			destination.gbc.gridheight = 1;
			destination.gbc.gridx = 0;
			destination.gbc.gridy++;
		}

		destination.gbc.anchor = GridBagConstraints.NORTHWEST;

		/* Do not stretch panels vertically!! */
		destination.gbc.fill = GridBagConstraints.HORIZONTAL;

		destination.add(panel, destination.gbc);
		
		
	}

	/**
	 * Retrieves or creates the menu with the given name in the menubar
	 *
	 * @param menuName  name of the menu
	 * @return JMenu
	 */
	protected JMenu getMenu(String menuName) {

		int numMenus = 0;
		boolean found = false;

		JMenu m = null;

		/* Create the menu bar (if not yet existing) */
		if (menubar_ == null) {
			menubar_ = new JMenuBar();
			menubarPanel_.addMenuBar(menubar_);
		}

		numMenus = menubar_.getMenuCount();

		for (int i = 0; i < numMenus; i++) {

			m = menubar_.getMenu(i);

			if (m.getText().equals(menuName)) {
				found = true;
				break;
			}
		}

		if (!found) {
			m = new JMenu(menuName);
			menubar_.add(m);
		}

		return m;
	}
	
	
	/**
	 * Add the display text action.
	 * 
	 * @param frame
	 * @param btnText
	 * @param title
	 * @param heading
	 */
	
	public void addDisplayTextAction(JFrame frame,String btnText,String title,String heading) {
		Action displayTextButtonAction = new DisplayTableAsTextAction(this,title,heading,frame); 
		displayTextButton_ = new JButton();
		displayTextButton_.setAction(displayTextButtonAction);
		displayTextButton_.setText(btnText);
	    addSummaryButton(displayTextButton_);
	}
	
	
	
	/**
	 * Set The text to be displayed on the display text button.
	 * The default is "Display Text" 
	 * @param title
	 */
	
	public void setDisplayTextButtonName(String name) {
		if(name == null || name.length() <= 0 ) {
			throw new IllegalArgumentException("Name must be freater than 0 in length");
		}
		displayTextButton_.setText(name);
	}
	




	/**
	 * Retrieve the column class information from the column chunks.
	 * This information should be used to choose the proper column
	 * renderer.
	 *
	 * @return Vector of column names and classes
	 */
	protected Vector getColumnChunkClasses() {

		Vector col_name_classes = new Vector();

		if (columnsPanel_ != null) {

			DbbSqlChunk columns[] = columnsPanel_.getActiveChunks();

			if (columns != null) {

				for (int i = 0; i < columns.length; i++) {
					col_name_classes.addElement(columns[i].getColumnTitle());
					col_name_classes.addElement(columns[i].getDataType());
				}
			}
		}
		try {
			DbbSqlChunk[] def_chunks = sqlEngine_.getDefaultColumnsChunk();

			for (int i = 0; i < def_chunks.length; i++) {
				col_name_classes.addElement(def_chunks[i].getColumnTitle());
				col_name_classes.addElement(def_chunks[i].getDataType());
			}
		} catch (Exception e) {
			stdlog_.warn("", e);
		}

		// for(int i=defaultChunks_.size()-1; i>=0; i--) {
		//     DbbSqlChunk chunk = (DbbSqlChunk)defaultChunks_.elementAt(i);
		//     col_name_classes.addElement(chunk.getColumnTitle());
		//     col_name_classes.addElement(chunk.getDataType());
		// }

		return col_name_classes;
	}

	/**
	 * Collect all the chunks from the widgets, pass them on to the
	 * SQL engine, and have it return the SQLquery text.
	 *
	 * @return Sql statement
	 */
	protected DbbSqlQueryChunks getQueryChunks() {

		DbbSqlQueryChunks query_chunks = new DbbSqlQueryChunks();

		/* Collect all the query_chunks chunks from the panels. */
		if (wherePanel_ != null) {
			//query_chunks.rows = wherePanel_.getActiveChunks();
			query_chunks.appendRows(wherePanel_.getActiveChunks());
			query_chunks.appendRows(wherePanel_.getAdditionalChunks());
		}

		if (columnsPanel_ != null) {
			query_chunks.columns = columnsPanel_.getActiveChunks();
			query_chunks.appendRows(columnsPanel_.getAdditionalChunks());
		}

		if (sortPanel_ != null)
			query_chunks.sort = sortPanel_.getActiveChunks();

		if (groupPanel_ != null)
			query_chunks.group = groupPanel_.getActiveChunks();

		/* Send the chunks to the sql engine to build the query. */
		return query_chunks;
	}

	/**
	  * Collect all the chunks from the widgets, pass them on to the
	  * SQL engine, and have it return the SQLquery text.
	  * This element search for second union  where elements.
	  *
	  * @return Sql statement
	  */
	protected DbbSqlQueryChunks getUnionChunks() {

		DbbSqlQueryChunks query_chunks = new DbbSqlQueryChunks();

		/* Collect all the query_chunks chunks from the panels. */
		if (whereUnionPanel_ != null) {
			//query_chunks.rows = wherePanel_.getActiveChunks();
			query_chunks.appendRows(whereUnionPanel_.getActiveChunks());
			query_chunks.appendRows(whereUnionPanel_.getAdditionalChunks());
		}

		if (columnsPanel_ != null) {
			query_chunks.columns = columnsPanel_.getActiveChunks();
			query_chunks.appendRows(columnsPanel_.getAdditionalChunks());
		}

		if (sortPanel_ != null)
			query_chunks.sort = sortPanel_.getActiveChunks();

		if (groupPanel_ != null)
			query_chunks.group = groupPanel_.getActiveChunks();

		/* Send the chunks to the sql engine to build the query. */
		return query_chunks;
	}

	/**
	 * Write a message in the message area with the number of the
	 * just retrieved rows
	 */
	public void updateDisplayedRowCount() {
		String msg = "Rows: " + summaryTable_.getRows();
		setMessage(msg);
		// stdlog_.debug("DbbView::updateDisplayedRowCount " + msg);
	}

	//----------------------------------------
	// Public methods
	//----------------------------------------

	/** @return The number of rows displayed in the summary table * @return int
	 */
	public int getRows() {
		return summaryTable_.getRows();
	}

	/**
	 * Add a default chunk to the associated sql engine. A default
	 * chunk is a chunk that has always to be considered when building
	 * the queries.
	 *
	 * @param chunk Default sql chunk
	 * @throws SQLException
	 * @throws RemoteException
	 * @throws MalformedURLException
	 * @throws NotBoundException
	 */
	public void addDefaultColumnsChunk(DbbSqlChunk chunk)
		throws
			SQLException,
			RemoteException,
			MalformedURLException,
			NotBoundException {

		sqlEngine_.addDefaultColumnsChunk(chunk);
		if (!defaultChunks_.contains(chunk)) {
			defaultChunks_.addElement(chunk);
		}
	}

	/**
	 * Add a default chunk to the associated sql engine. A default
	 * chunk is a chunk that has always to be considered when building
	 * the queries.
	 *
	 * @param chunk     Default sql chunk
	 * @param visible   Boolean flag to prevent the view to show the
	 *                  column associated to the default sql chunk
	 * @throws SQLException
	 * @throws RemoteException
	 * @throws MalformedURLException
	 * @throws NotBoundException
	 */
	public void addDefaultColumnsChunk(DbbSqlChunk chunk, boolean visible)
		throws
			SQLException,
			RemoteException,
			MalformedURLException,
			NotBoundException {
		addDefaultColumnsChunk(chunk);
		if (!visible)
			summaryTable_.hideColumn(chunk.getLabelWithoutQuotes());
	}

	/**
	 * Set a column selection panel
	 *
	 * @param panel     DbbPanel defing a column selection panel
	 * @param position  Relative position in the view (NSEW)
	 */
	public void setColumnsPanel(DbbPanel panel, String position) {
		setColumnsPanel(panel, position, true, -1, -1);
	}

	/**
	 * Set a column selection panel
	 *
	 * @param panel     DbbPanel defing a column selection panel
	 * @param position  Relative position in the view (NSEW)
	 * @param visible   specifies if the panel is visible
	 */
	public void setColumnsPanel(
		DbbPanel panel,
		String position,
		boolean visible) {
		setColumnsPanel(panel, position, visible, -1, -1);
	}

	/**
	 * Set a column selection panel
	 *
	 * @param panel     DbbPanel defing a column selection panel
	 * @param position  Relative position in the view (NSEW)
	 * @param visible   specifies if the panel is visible
	 * @param x         x position in the container grid
	 * @param y         y position in the container grid
	 */
	public void setColumnsPanel(
		DbbPanel panel,
		String position,
		boolean visible,
		int x,
		int y) {
		//I CHANGED THIS CODE!!!!
	//	boolean flag = true;
		if (columnsPanel_ != null) removeJPanel(columnsPanel_, position);
		
		addJPanel(panel, position, x, y);
			
		columnsPanel_ = panel;
		columnsPanel_.setVisible(visible);
		//END OF MODIFY

		// setting column checkboxes off also switches off the associated sort column
		DbbWidget[] selectWgts = columnsPanel_.getAllWidgets();
		for (int i = 0; i < selectWgts.length; i++) {
			try {
				final String selectCol =
					selectWgts[i].getSqlChunk().getColumn();
				final DbbSqlTable selectTbl =
					selectWgts[i].getSqlChunk().getTable();
				selectWgts[i].addListener(new DbbWidgetListener() {
					public void dbbWidgetAction(DbbWidgetEvent ev) {
						DbbCheckBox cb = (DbbCheckBox) ev.getSource();
						String cbVal = cb.get();
						if (cbVal.equals("0")
							|| cbVal.equalsIgnoreCase("false") && sortPanel_ != null) {
							DbbWidget[] sortWgts = sortPanel_.getAllWidgets();
							for (int j = 0; j < sortWgts.length; j++) {
								try {
									String sortCol =
										sortWgts[j].getSqlChunk().getColumn();
									DbbSqlTable sortTbl =
										sortWgts[j].getSqlChunk().getTable();
									if (sortCol.equals(selectCol)
										&& sortTbl == selectTbl) {
										sortWgts[j].set("0");
										sortWgts[j].fireDbbWidgetAction();
										// return;
									}
								} catch (DbbInvalidSqlOperatorType e) {
									e.printStackTrace();
								}
							}
						}
					}
				});
			} catch (DbbInvalidSqlOperatorType e) {
				e.printStackTrace();
			}
		}
	}
	/**
	 * New add!!
	 * @param panel
	 */
	protected void removeJPanel(JPanel panel, String position) {

		/* Wrong vales: do nothing */
		if (position == null || panel == null)
			return;

		DbbViewContainer destination;

		String str = position.toUpperCase().trim();
		char p = str.charAt(0);

		/* Check the position in the view */
		switch (p) {
			case 'N' :
				destination = northPanel_;
				break;
			case 'S' :
				destination = southPanel_;
				break;
			case 'E' :
				destination = eastPanel_;
				break;
			case 'W' :
				destination = westPanel_;
				break;
			default :
				return;
		}
		panel.setVisible(false);
		destination.remove(panel);
	}

	/**
	 * Set a column selection panel
	 *
	 * @param panel     DbbPanel defing a column selection panel
	 * @param position  Relative position in the view (NSEW)
	 * @param visible   specifies if the panel is visible
	 * @param x         x position in the container grid
	 * @param y         y position in the container grid
	 */
	public void setColumnsPanelFull(
		DbbPanel panel,
		String position,
		boolean visible,
		int x,
		int y) {
		columnsPanelFull_ = panel;
		setColumnsPanel(panel, position, visible, x, y);
	}

	/**
	 * Set a row selection panel (delegated to define a where clause)
	 *
	 * @param panel     DbbPanel defing a row selection panel
	 * @param position  Relative position in the view (NSEW)
	 */
	public void setWherePanel(DbbPanel panel, String position) {
		setWherePanel(panel, position, true, -1, -1);
	}

	/**
	 * Set a row selection panel (delegated to define a where clause)
	 *
	 * @param panel     DbbPanel defing a row selection panel
	 * @param position  Relative position in the view (NSEW)
	 * @param visible   specifies if the panel is visible
	 */
	public void setWherePanel(
		DbbPanel panel,
		String position,
		boolean visible) {
		setWherePanel(panel, position, visible, -1, -1);
	}

	/**
	 * Set a row selection panel (delegated to define a where clause)
	 *
	 * @param panel     DbbPanel defing a row selection panel
	 * @param position  Relative position in the view (NSEW)
	 * @param visible   specifies if the panel is visible
	 * @param x         x position in the container grid
	 * @param y         y position in the container grid
	 */
	public void setWherePanel(
		DbbPanel panel,
		String position,
		boolean visible,
		int x,
		int y) {
		//wherePanel_ = panel;
		//addJPanel(wherePanel_, position, x, y);
		//wherePanel_.setVisible(visible);
		//I CHANGED THIS CODE!!!!
		if (wherePanel_ != null) removeJPanel(wherePanel_, position);
		
		addJPanel(panel, position, x, y);
			
		wherePanel_ = panel;
		wherePanel_.setVisible(visible);
		//END OF MODIFY
	}
	
	public void setWherePanelFull(
			DbbPanel panel,
			String position,
			boolean visible,
			int x,
			int y) {
			rowsPanelFull_ = panel;
			setWherePanel(panel, position, visible);
			/*wherePanel_ = panel;
			addJPanel(wherePanel_, position, x, y);
			wherePanel_.setVisible(visible);*/
		}

	/**
	 * Set a sort panel (delegated to define the sorting criteria)
	 * @param panel     DbbPanel defing a sort panel
	 * @param position  Relative position in the view (NSEW)
	 */
	public void setSortPanel(DbbPanel panel, String position) {
		setSortPanel(panel, position, true, -1, -1);
	}
	
	public void setSortPanelFull(
			DbbPanel panel,
			String position,
			boolean visible,
			int x,
			int y) {
			sortPanelFull_ = panel;
			setSortPanel(panel, position, visible);
			/*wherePanel_ = panel;
			addJPanel(wherePanel_, position, x, y);
			wherePanel_.setVisible(visible);*/
		}

	/**
	 * Set a sort panel (delegated to define the sorting criteria)
	 * @param panel     DbbPanel defing a sort panel
	 * @param position  Relative position in the view (NSEW)
	 * @param visible   specifies if the panel is visible
	 */
	public void setSortPanel(
		DbbPanel panel,
		String position,
		boolean visible) {
		setSortPanel(panel, position, visible, -1, -1);
	}

	/**
	 * Set a sort panel (delegated to define the sorting criteria)
	 * @param panel     DbbPanel defing a sort panel
	 * @param position  Relative position in the view (NSEW)
	 * @param visible   specifies if the panel is visible
	 * @param x         x position in the container grid
	 * @param y         y position in the container grid
	 */
	public void setSortPanel(
		DbbPanel panel,
		String position,
		boolean visible,
		int x,
		int y) {
// ORIGINAL CODE	
		/*sortPanel_ = panel;
		addJPanel(panel, position, x, y);
		panel.setVisible(visible);*/
//		NEW CODE!!!!
		if (sortPanel_ != null) removeJPanel(sortPanel_, position);
		
		addJPanel(panel, position, x, y);
			
		sortPanel_ = panel;
		sortPanel_.setVisible(visible);
		
		

		// clicking sort checkboxes causes sets the associated select column
		DbbWidget[] sortWgts = sortPanel_.getAllWidgets();
		for (int i = 0; i < sortWgts.length; i++) {
			try {
				final String sortCol = sortWgts[i].getSqlChunk().getColumn();
				final DbbSqlTable sortTbl =
					sortWgts[i].getSqlChunk().getTable();
				sortWgts[i].addListener(new DbbWidgetListener() {
					public void dbbWidgetAction(DbbWidgetEvent ev) {
						if (ev.getSource().getClass()
							== DbbArrowButton.class) {
							DbbArrowButton ab = (DbbArrowButton) ev.getSource();
							if (!ab.get().equals("0")
								&& columnsPanel_ != null) {
								DbbWidget[] colWgts =
									columnsPanel_.getAllWidgets();
								for (int j = 0; j < colWgts.length; j++) {
									try {
										String selectCol =
											colWgts[j]
												.getSqlChunk()
												.getColumn();
										DbbSqlTable selectTbl =
											colWgts[j].getSqlChunk().getTable();
										if (sortCol.equals(selectCol)
											&& sortTbl == selectTbl) {
											colWgts[j].set("1");
											colWgts[j].fireDbbWidgetAction();
											// return;
										}
									} catch (DbbInvalidSqlOperatorType e) {
										e.printStackTrace();
									}
								}
							}
						}
					}
				});
			} catch (DbbInvalidSqlOperatorType e) {
				e.printStackTrace();
			}
		}
	}

	/**
	 * @return The sort panel
	 */
	public DbbPanel getSortPanel() {
		return sortPanel_;
	}

	/**
	 * @return The columns selection panel
	 */
	// AMC, 10-Sep-2003: added
	public DbbPanel getColumnsPanel() {
		return columnsPanel_;
	}

	/**
	 * @return The rows selection panel
	 * @deprecated.
	 * Use getRowsPanel().
	 */
	// AMC, 10-Sep-2003: added
	public DbbPanel getWherePanel() {
		return wherePanel_;
	}
	
	public DbbPanel getRowsPanel() {
		return wherePanel_;
	}

	/**
	 * Set a group panel (delegated to define the group criteria)
	 *
	 * @param panel     DbbPanel defing a sort panel
	 * @param position  Relative position in the view (NSEW)
	 */
	public void setGroupPanel(DbbPanel panel, String position) {
		setGroupPanel(panel, position, true, -1, -1);
	}

	/**
	 * Set a group panel (delegated to define the group criteria)
	 *
	 * @param panel     DbbPanel defing a sort panel
	 * @param position  Relative position in the view (NSEW)
	 * @param visible   specifies if the panel is visible
	 * @param x         x position in the container grid
	 * @param y         y position in the container grid
	 */
	public void setGroupPanel(
		DbbPanel panel,
		String position,
		boolean visible,
		int x,
		int y) {
		groupPanel_ = panel;
		addJPanel(panel, position, x, y);
		panel.setVisible(visible);
	}

	/**
	 * Add a view panel and display it at a specific position
	 *
	 * @param panel     View panel to be displayed
	 * @param position  Relative position in the view (NSEW)
	 */
	public void addPanel(DbbPanel panel, String position) {
		this.addJPanel(panel, position, -1, -1);
	}

	/**
	 * Add a view panel and display it at a specific position
	 *
	 * @param panel     View panel to be displayed
	 * @param position  Relative position in the view (NSEW)
	 * @param x         X position in the container grid
	 * @param y         Y position in the container grid
	 */
	public void addPanel(DbbPanel panel, String position, int x, int y) {
		this.addJPanel(panel, position, x, y);
	}

	/**
	 * Add a detail view panel and display it at a specific position
	 *
	 * @param detail_view       View panel to be displayed
	 * @param position          Relative position in the view (NSEW)
	 */
	public void setDetailView(JPanel detail_view, String position) {
		this.addJPanel(detail_view, position, -1, -1);
		detail_ = detail_view;
	}

	/**
	 * @return The detail view reference
	 */
	public JPanel getDetailView() {
		return detail_;
	}

	/**
	 * Method setMenuBar
	 * @param menu_bar JMenuBar
	 */
	public void setMenuBar(JMenuBar menu_bar) {
		menubar_ = menu_bar;
		menubarPanel_.addMenuBar(menubar_);
	}

	/**
	 * Adds a action to the named menu
	 *
	 * @param menuName  name of the menu
	 * @param action    action to be added
	 * @return JMenuItem
	 */
	public JMenuItem addMenuItem(String menuName, Action action) {
		JMenu m = getMenu(menuName);
		JMenuItem item = m.add(action);
		return item;
	}

	/**
	 * Adds a gui component to the named menu
	 *
	 * @param menuName  name of the menu
	 * @param comp      component to be added
	 */
	public void addMenuComponent(String menuName, Component comp) {
		JMenu m = getMenu(menuName);
		m.add(comp);
	}

	/**
	 * Adds a action to the named menu
	 *
	 * @param menuName  name of the menu
	 * @param subMenu   subMenu to be added
	 * @return JMenuItem
	 */
	public JMenuItem addSubMenu(String menuName, JMenuItem subMenu) {
		JMenu m = getMenu(menuName);
		JMenuItem mnItem = m.add(subMenu);
		return mnItem;
	}

	/**
	 * Adds a separator line to the specified menu
	 *
	 * @param menuName  Menu name
	 */
	public void addSeparator(String menuName) {
		JMenu menu = getMenu(menuName);
		menu.addSeparator();
	}

	/**
	 * Add an optional button to the summary button panel
	 *
	 * @param new_button A new JButton object created by the end application
	 */
	public void addSummaryButton(JButton new_button) {
		summaryButtons_.addButton(new_button);
	}

	/** Notification mechanism */
	public void addNotify() {
		super.addNotify();
		frame_ = (JFrame) getTopLevelAncestor();
		glassPane_ = (JPanel) frame_.getGlassPane();
		mouseListener_ = new PanelMouseListener();
		glassPane_.addMouseListener(mouseListener_);
	}

	/** Set the internal inProgress_ boolean variable * @param inProgress boolean
	 */
	public void setInProgress(boolean inProgress) {
		inProgress_ = inProgress;
	}

	/**
	 * Set the container glass panel to visible
	 *
	 * @param visible Boolean flag
	 */
	public void setGlassPaneVisible(boolean visible) {
		glassPane_.setVisible(visible);
	}

	/** Hide the count button */
	public void hideCountButton() {
		summaryButtons_.hideCountButton();
	}

	/** Hide the stop button */
	public void hideStopButton() {
		summaryButtons_.hideStopButton();
	}

	/** Hide the reset button */
	public void hideResetButton() {
		summaryButtons_.hideResetButton();
	}

	/**
	 * Hide the "Sort" button in the summary button panel
	 */
	public void hideSortButton() {
		summaryButtons_.hideSortButton();
	}

	/**
	  * Hide the "Sort" button in the summary button panel
	  */
	public void hideQueryButton() {
		summaryButtons_.hideQueryButton();
	}

	/** @return summaryButtons_ attribute * @return DbbButtonPanel
	 */
	public DbbButtonPanel getSummaryButtons() {
		return summaryButtons_;
	}

	// private javax.swing.Timer messageTimer_ = null;

	// private static final int MSG_TIMEOUT_ = 5000;

	/**
	 * Method setMessage
	 * @param s String
	 */
	public void setMessage(String s) {
		// if (messageTimer_ == null) {
		//     ActionListener invoke_listener = new ActionListener() {
		//         public void actionPerformed(ActionEvent event){
		//             message_.setText("");
		//             messageTimer_.stop();
		//         }
		//     };
		//     messageTimer_ = new Timer(MSG_TIMEOUT_, invoke_listener);
		// }
		message_.setText(s);
		// messageTimer_.start();
	}
	
	/**
	 * @return String
	 */
	public String getMessage() {
	   return message_.getText();
	}

	/**
	 * Field enableFlag_
	 */
	private boolean enableFlag_ = true;

	/**
	 * Method getEnabled
	 * @return boolean
	 */
	public boolean getEnabled() {
		return enableFlag_;
	}

	/**
	 * Enable or disable the view
	 *
	 * @param enable Boolean flag to enable or disable the view
	 */
	public void setEnabled(boolean enable) {
		enableFlag_ = enable;
		setInProgress(!enable);
		setGlassPaneVisible(!enable);
		summaryButtons_.setEnabled(enable);
		if (!enable)
			glassPane_.requestFocus();
	}

	//----------------------------------------
	// Selection methods
	//----------------------------------------

	/**
	 * @return  array of integers containing indexes of selected rows
	 */
	public int[] getSelectedRows() {
		return summaryTable_.getSelectedRows();
	}

	/**
	 * Method getTable
	 * @return DbbTable
	 */
	public DbbTable getTable() {
		return summaryTable_;
	}

	/** @return The index of the selected row. * @return int
	 */
	public int getSelectedRow() {
		return summaryTable_.getSelectedRow();
	}

	/**
	 * Select a given row in the table
	 *
	 * @param row Index of the row to be selected
	 */
	public void setSelectedRow(int row) {
		summaryTable_.setSelectedRow(row);
	}

	/**
	 * Deselect a given row in the table
	 *
	 * @param row Index of the row to be deselected
	 */
	public void deselectRow(int row) {
		summaryTable_.deselectRow(row);
	}

	/** Select all the rows in the table */
	public void selectAllRows() {
		summaryTable_.selectAllRows();
	}

	/** Deselect any row  and column in the table */
	public void clearSelection() {
		summaryTable_.clearSelection();
	}

	/**
	 * Get a value contained in a specific table cell
	 *
	 * @param row Row index
	 * @param column Column index
	 * @return Object
	 */
	public Object getValueAt(int row, int column) {
		return summaryTable_.getValueAt(row, column);
	}

    /**
     * Get a value contained in a specific table cell
     *
     * @param row Row index
     * @param column_name Column name
     * @return Object
     */
    public Object getValueAt(int row, String column_name) {
        return summaryTable_.getValueAt(row, column_name);
    }  
    
   
	//----------------------------------------
	// Event methods
	//----------------------------------------

	/**
	 * List of listeners to be notified when firing a DbbQueryTerminated event
	 */
	protected EventListenerList listenerList_ = new EventListenerList();

	/** Last fired DbbQueryTerminated object */
	protected DbbQueryTerminated queryTerminatedEvent_ = null;
	/**
	 * Field querySortedEvent_
	 */
	protected DbbSortTerminated querySortedEvent_ = null;

	/**
	 * Add a listener to the listeners' list. This listener will be
	 * notified when firing an event of the DbbSortTerminated class.
	 *
	 * @param l DbbSortTerminatedListener reference
	 */
	public void addSortTerminatedListener(DbbSortTerminatedListener l) {
		listenerList_.add(DbbSortTerminatedListener.class, l);
	}

	/**
	 * Add a listener to the listeners' list. This listener will be
	 * notified when firing an event of the DbbSortTerminated class.
	 *
	 * @param l DbbSortTerminatedListener reference
	 */
	public void removeSortTerminatedListener(DbbSortTerminatedListener l) {
		listenerList_.add(DbbSortTerminatedListener.class, l);
	}

	/**
	 * Add a listener to the listeners' list. This listener will be
	 * notified when firing an event of the DbbQueryTerminated class.
	 *
	 * @param l DbbQueryTerminatedListener reference
	 */
	public void addQueryTerminatedListener(DbbQueryTerminatedListener l) {
		listenerList_.add(DbbQueryTerminatedListener.class, l);
	}

	/**
	 * Remove a listener from the listeners' list
	 *
	 * @param l DbbQueryTerminatedListener reference
	 */
	public void removeQueryTerminatedListener(DbbQueryTerminatedListener l) {
		listenerList_.remove(DbbQueryTerminatedListener.class, l);
	}

	/**
	 * Notify all listeners that have registered interest for
	 * notification on this event type. The event instance is lazily
	 * created using the parameters passed into the fire method.
	 */
	public void fireQueryTerminatedAction() {

		// Guaranteed to return a non-null array
		Object[] listeners = listenerList_.getListenerList();
		
		// Process the listeners last to first, notifying
		// those that are interested in this event
		for (int i = listeners.length - 2; i >= 0; i -= 2) {

			if (listeners[i] == DbbQueryTerminatedListener.class) {

				// Lazily create the event:
				if (queryTerminatedEvent_ == null)
					queryTerminatedEvent_ = new DbbQueryTerminated(this);

				DbbQueryTerminatedListener l =
					((DbbQueryTerminatedListener) listeners[i + 1]);

				l.queryTerminatedAction(queryTerminatedEvent_);
			}
		}
		
		
	}

	/**
	 * Notify all listeners that have registered interest for
	 * notification on this event type. The event instance is lazily
	 * created using the parameters passed into the fire method.
	 */
	public void fireQuerySortedAction() {

		// Guaranteed to return a non-null array
		Object[] listeners = listenerList_.getListenerList();

		// Process the listeners last to first, notifying
		// those that are interested in this event
		for (int i = listeners.length - 2; i >= 0; i -= 2) {

			if (listeners[i] == DbbSortTerminatedListener.class) {

				// Lazily create the event:
				if (querySortedEvent_ == null)
					querySortedEvent_ = new DbbSortTerminated(this);

				DbbSortTerminatedListener l =
					((DbbSortTerminatedListener) listeners[i + 1]);

				l.sortTerminatedAction(querySortedEvent_);
			}
		}
	}

	/**
	 * Add a selection listener to the list of listeners to be
	 * notified when selecting table rows
	 *
	 * @param listener List selection listener
	 */
	public void addListSelectionListener(ListSelectionListener listener) {
		ListSelectionModel sel_model =
			summaryTable_.getTable().getSelectionModel();
		sel_model.addListSelectionListener(listener);
	}

	/**
	 * Add a selection listener to the list of listeners to be
	 * notified when selecting table rows
	 *
	 * @param listener List selection listener
	 */
	public void removeListSelectionListener(ListSelectionListener listener) {
		ListSelectionModel sel_model =
			summaryTable_.getTable().getSelectionModel();
		sel_model.removeListSelectionListener(listener);
	}

	/**
	 * Add rows to the view's inner table
	 *
	 * @param result Sql result coming from the server
	 */
	public void addRows(DbbSqlQueryResult result) {

		/* Override the default column type with the display types
		 * specified in the column selection chunks
		 */
		if (!columnClassesDefined_) {
			/* Test if no rows have been returned. */
			if (result == null) {
				updateDisplayedRowCount();
				return;
			}

			result.updateColumnTypes(this.getColumnChunkClasses());
			columnClassesDefined_ = true;
		}

		/* Update the display table with the new results. */
		summaryTable_.addRows(result);
		updateDisplayedRowCount();
	}

	/**
	 * SQL start event handler: start the sql engine query mechanism
	 *
	 * @param event     DbbQueryStarted event
	 * @see org.eso.ohs.core.dbb.sql.DbbSqlListener#sqlStartAction
	 * (DbbQueryStarted)
	 */
	public synchronized void sqlStartAction(DbbQueryStarted event) {
		boolean result = preConditionCheckFields ();
		summaryTable_.getHeaderRenderer().resetState();
		if (result) {
		
			if (!isThreadCompleted()) {
				incrementCounter();
				return;
			}
	
			setEnabled(false);
			summaryTable_.clear();
			//    columnsPanel_.setEnabled(false);
			setMessage("Searching...");
	
			columnClassesDefined_ = false;
	
			//try {
	
			setEnabled(false);
			ioThread_ = new DBIOThread(this);
			counter_ = 1;
			ioThread_.start();
			setThreadCompleted(false);
		
		}
	}


	/**
	 * @return boolean true, false
	 */
	private boolean preConditionCheckFields() {
		return getVerifier().verify();
	}

	/**
	 * SQL count event handler: start the sql engine count mechanism
	 *
	 * @param event     DbbCountStarted event
	 * @see org.eso.ohs.core.dbb.sql.DbbSqlListener#sqlCountAction
	 * (DbbCountStarted)
	 */
	public void sqlCountAction(DbbCountStarted event) {
		try {
			setMessage("Counting rows...");
			int row_number = sqlEngine_.execSqlCount(this.getQueryChunks());
			setMessage(
				"Pre-counted " + row_number + " rows (including duplicates)");
		} catch (SQLException e) {
			stdlog_.warn("", e);
		} catch (MalformedURLException e) {
			stdlog_.warn("", e);
		} catch (NotBoundException e) {
			stdlog_.warn("", e);
		} catch (RemoteException e) {
			stdlog_.warn("", e);
		}
	}

	/**
	 * SQL stop event handler: ask the sql engine to stop the query execution
	 *
	 * @param event     DbbQueryStopped event
	 * @see org.eso.ohs.core.dbb.sql.DbbSqlListener#sqlStopAction
	 * (DbbQueryStopped)
	 */
	public void sqlStopAction(DbbQueryStopped event) {
		if (ioThread_ != null) {
			if (!isThreadCompleted()) {
				ioThread_.stopQueryThread();
			}
		}
		
		setEnabled(true);
		setThreadCompleted(false);
		String msg = "Query aborted after " + summaryTable_.getRows() + " rows";
		setMessage(msg);
		stdlog_.debug(msg);
	}

	/**
	 * SQL reset event handler: reset all the panels
	 *
	 * @param event     DbbQueryReset event
	 * @see org.eso.ohs.core.dbb.sql.DbbSqlListener#sqlResetAction
	 * (DbbQueryReset)
	 */
	public void sqlResetAction(DbbQueryReset event) {
		if (wherePanel_ != null)
			wherePanel_.reset();
		if (whereUnionPanel_ != null)
			whereUnionPanel_.reset();
		sqlResetColSortAction(event);
	}

	/**
	  * SQL reset event handler: reset sort and columns panels
	  *
	  * @param event     DbbQueryReset event
	  * @see org.eso.ohs.core.dbb.sql.DbbSqlListener#sqlResetAction
	  * (DbbQueryReset)
	  */
	public void sqlResetColSortAction(DbbQueryReset event) {
		/* Check for preconditions */
		if (columnsPanel_ == null) throw new IllegalArgumentException(MsgManager.errContract("The Columns Panel is null"));
		if (sortPanel_ == null) throw new IllegalArgumentException(MsgManager.errContract("The Sorting Panel is null"));
		/* DbC */	
		
		columnsPanel_.reset();
		sortPanel_.reset();
	}
	/**
	  * SQL reset event handler: reset selection criteria panel (also known as rows or where panel)
	  *
	  * @param event     DbbQueryReset event
	  * @see org.eso.ohs.core.dbb.sql.DbbSqlListener#sqlResetAction
	  * (DbbQueryReset)
	  */
	public void sqlResetRowsAction(DbbQueryReset event) {
		/* Check for preconditions */
		if (wherePanel_ == null) throw new IllegalArgumentException(MsgManager.errPrecondition("The Rows Panel (Where panel) is null"));
		/* DbC */	
			
		wherePanel_.reset();
	}
	/**
	  * SQL reset event handler: reset sorting criteria panel (also known as sort or sorting panel)
	  *
	  * @param event     DbbQueryReset event
	  * @see org.eso.ohs.core.dbb.sql.DbbSqlListener#sqlResetAction
	  * (DbbQueryReset)
	  */
	public void sqlResetSortingAction(DbbQueryReset event) {
		/* Check for preconditions */
		if (sortPanel_ == null) throw new IllegalArgumentException(MsgManager.errPrecondition("The Sorting Panel is null"));
		/* DbC */	
		
		sortPanel_.reset();
	}
	/**
	 * Table sort event handler: start table sorting according to the
	 * condition found in the sorting panel
	 *
	 * @param event DbbTableSortEvent event
	 * @see org.eso.ohs.core.dbb.client.DbbTableListener#tableSortAction
	 * (DbbTableSortEvent)
	 */
	public void tableSortAction(DbbTableSortEvent event) {

		String msg = "Sorting by ";
		DbbSqlChunk sort[] = null;

		if (sortPanel_ == null) {
			return;
		}

		sort = sortPanel_.getActiveChunks();

		/* Retrieve also the column chunks because they contain the label
		 * and the position (in the table model) of the column to be sorted
		 */
		Vector colIndexes = new Vector();
		Vector criteria = new Vector();

		/* Check all the sort column names. */
		for (int i = 0; i < sort.length; i++) {
			Boolean val;
			String label = sort[i].getLabelWithoutQuotes();

			DbbSqlOperator op = sort[i].getOperator();
			String op_val = null;

			if (op != null) {
				op_val = op.get();
			} else {
				stdlog_.debug(
					"Operator is not set for chuck having label: " + label);
			}

			if (op_val != null && op_val.equals("descending")) {
				val = new Boolean(false);
			} else {
				val = new Boolean(true);
			}

			colIndexes.addElement(label);

			msg = msg + label + ", ";

			criteria.addElement(val);

			stdlog_.debug("Columns to sort: " + label + " " + op_val);

		}

		/* Sort the table only if there are valid columns to be sorted */
		if (colIndexes.size() > 0) {

			msg = msg.substring(0, msg.length() - 2) + "...";
			setMessage(msg);

			stdlog_.debug(msg);
			/* Start sorting */
			summaryTable_.sort(colIndexes, criteria);

			msg = "Sorted " + summaryTable_.getRows() + " rows";
			setMessage(msg);

		} else {

			msg = "Nothing to sort.";
			setMessage(msg);
		}

		fireQuerySortedAction();
	}

	/** @return A string with the table contents * @return String
	 */
	public String printTable() {
		return summaryTable_.printTable();
	}

	/** Delegate the summary table to fire a query started event */
	public void fireDbbQueryStartedAction() {
		summaryButtons_.fireDbbQueryStartedAction();
	}

	//----------------------------------------
	// Internal Classes
	//----------------------------------------

	/**
	 * Internal class to define the specific DbbPanel containers located
	 * all around the DbbTable. These objects are JPanels with grid bag
	 * LayoutManagers, storing the grid bag constraints in order to
	 * allow DbbPanel packing.
	 * @author ddorigo
	 */
	private static class DbbViewContainer extends JPanel {

		/**
		 * Field gbc
		 */
		public GridBagConstraints gbc = new GridBagConstraints();

		/**
		 * Constructor for DbbViewContainer
		 */
		public DbbViewContainer() {
			super(true);
			setOpaque(true);
			setLayout(new GridBagLayout());
			gbc.gridx = 0;
			gbc.gridy = 0;
			gbc.fill = GridBagConstraints.HORIZONTAL;
			gbc.anchor = GridBagConstraints.NORTHWEST;

			gbc.weightx = 1;
			gbc.weighty = 1;

		}

		/**
		 * Method add
		 * @param c Component
		 * @param o Object
		 */
		public void add(Component c, Object o) {
			super.add(c, o);
		}
	}

	/**
	 * Internal class to define the specific MenuBarPanel container
	 * @author ddorigo
	 */
	private static class MenuBarPanel extends JPanel {

		/**
		 * Field gbc_
		 */
		protected GridBagConstraints gbc_;

		/**
		 * Constructor for MenuBarPanel
		 */
		public MenuBarPanel() {

			super(true);

			setLayout(new GridBagLayout());
			setOpaque(true);

			gbc_ = new GridBagConstraints();
			gbc_.insets = new Insets(0, 0, 0, 0);

			gbc_.gridx = 0;
			gbc_.gridy = 0;

			// Display components within the cells
			gbc_.fill = GridBagConstraints.HORIZONTAL;

			gbc_.weightx = 10;
			gbc_.weighty = 0.1;

			gbc_.gridwidth = GridBagConstraints.REMAINDER;
			gbc_.gridheight = 1;
		}

		/**
		 * Method addMenuBar
		 * @param mb JMenuBar
		 */
		public void addMenuBar(JMenuBar mb) {
			this.add(mb, gbc_);
		}
	}

	/**
	 * Table Mouse Listener to make sure data is saved to the business objects
	 * @author ddorigo
	 */
	private class PanelMouseListener implements MouseListener {

		/**
		 * Field stopButton_
		 */
		private JButton stopButton_;

		/**
		 * Constructor for PanelMouseListener
		 */
		public PanelMouseListener() {
			stopButton_ = summaryButtons_.getStopButton();
		}

		/**
		 * Method stopButtonClicked
		 * @param p Point
		 * @return boolean
		 */
		private boolean stopButtonClicked(Point p) {
			Dimension butDimension = stopButton_.getSize();
			Point buttonCoords =
				SwingUtilities.convertPoint(glassPane_, p, stopButton_);

			if (buttonCoords.x < 0
				|| buttonCoords.y < 0
				|| buttonCoords.x > butDimension.width
				|| buttonCoords.y > butDimension.height)
				return false;
			else
				return true;
		}

		/**
		 * Method consume
		 * @param e MouseEvent
		 */
		private void consume(MouseEvent e) {
			if (inProgress_) {
				if (stopButtonClicked(e.getPoint())) {
					glassPane_.setVisible(false);
					stopButton_.doClick();
				}
			}
		}

		/**
		 * Mouse clicked event
		 *
		 * @param e mouse clicked event
		 * @see java.awt.event.MouseListener#mouseClicked(MouseEvent)
		 */
		public void mouseClicked(MouseEvent e) {
			consume(e);
		}

		/**
		 * Mouse pressed event
		 *
		 * @param e mouse pressed event
		 * @see java.awt.event.MouseListener#mousePressed(MouseEvent)
		 */
		public void mousePressed(MouseEvent e) {
			// consume(e);
		}

		/**
		 * Mouse released event
		 *
		 * @param e mouse released event
		 * @see java.awt.event.MouseListener#mouseReleased(MouseEvent)
		 */
		public void mouseReleased(MouseEvent e) {
			// consume(e);
		}

		/**
		 * Mouse entered event
		 *
		 * @param e mouse entered event
		 * @see java.awt.event.MouseListener#mouseEntered(MouseEvent)
		 */
		public void mouseEntered(MouseEvent e) {
			// consume(e);
		}

		/**
		 * Mouse exited event
		 *
		 * @param e mouse exited event
		 * @see java.awt.event.MouseListener#mouseExited(MouseEvent)
		 */
		public void mouseExited(MouseEvent e) {
			// consume(e);
		}
	}

	/**
	 * Field isThreadCompleted_
	 */
	private boolean isThreadCompleted_ = true;

	/**
	 * Method setThreadCompleted
	 * @param val boolean
	 */
	private void setThreadCompleted(boolean val) {
		isThreadCompleted_ = val;
	}

	/**
	 * Method queryFailed
	 */
	protected void queryFailed() {
		stdlog_.error("database query failed");
		setEnabled(true);
		setMessage("");
		// fireQueryTerminatedAction();
		JOptionPane.showMessageDialog(
			frame_,
			"A Database error has occured.\n"
				+ "Please inform your Database Administrator",
			"Database Error",
			JOptionPane.ERROR_MESSAGE);
	}

	/**
	 * Method isThreadCompleted
	 * @return boolean
	 */
	private synchronized boolean isThreadCompleted() {
		return isThreadCompleted_;
	}

	/**
	 * Method incrementCounter
	 */
	private void incrementCounter() {
		counter_++;
	}

	/**
	 * Method decrementCounter
	 * @return int
	 */
	private int decrementCounter() {
		return --counter_;
	}

	/**
	 * This internal class defines thread objects to run the database
	 *  browser queries
	 * @author ddorigo
	 */
	protected class DBIOThread extends Thread {
		/**
		 * Field view_
		 */
		protected DbbView view_;

		/**
		 * Field breakQueryThreadFlag_
		 */
		private boolean breakQueryThreadFlag_ = false;

		/**
		 * Constructor for DBIOThread
		 * @param view DbbView
		 */
		public DBIOThread(DbbView view) {
			view_ = view;
			setPriority(getPriority() - 1);
		}

		/**
		 * Stop the current running thread and set the "query stopped"
		 * boolean flag
		 */
		public synchronized void stopQuery() {
			breakQueryThreadFlag_ = true;
			//try {
			//  wait();
			//}
			//catch(InterruptedException e) {
			//    stdlog_.warn("",e);
			//}
		}

		/**
		 * Stop the current running thread and set the "query stopped"
		 * boolean flag
		 */
		public synchronized void stopQueryThread() {
			breakQueryThreadFlag_ = true;
			// try {
			//     wait();
			// } catch(InterruptedException e) {
			//     stdlog_.warn("", e);
			// }
		}

		/**
		 * author amchavan, 28-Aug-2001: fixed 1389
		 * @see java.lang.Runnable#run()
		 */
		public void run() {

			boolean ok;
			while (true) {
				ok = false;
				try {
					summaryTable_.clear();
					columnClassesDefined_ = false;
					long start_time = System.currentTimeMillis();
					final DbbQuery qobj = runQuery();
					while (qobj.hasMoreRows()) {
						if (breakQueryThreadFlag_) {
							stdlog_.debug(
								"Query stopped, close the result set cursor");
							break;
						}

						qobj.readRows();

						/* This method should be used when an application
						 * thread needs to update the GUI
						 */
						SwingUtilities.invokeAndWait(new Runnable() {
							public void run() {
								try {
									view_.addRows(qobj.getResult());
									view_.updateDisplayedRowCount();
								} catch (RemoteException e) {
									stdlog_.warn("", e);
								}
							}
						});
					}

					SwingUtilities.invokeAndWait(new Runnable() {
						public void run() {
							view_.setEnabled(true);
							view_.updateDisplayedRowCount();
							// Fire a query terminated event
							view_.fireQueryTerminatedAction();
						}
					});

					if (stdlog_.isDebugEnabled()) {
						long end_time = System.currentTimeMillis();
						long diff_time = end_time - start_time;

						// @author amchavan, 28-Aug-2001: fixed 1389
						SimpleDateFormat format =
							new SimpleDateFormat("mmm:ss.SSS");
						stdlog_.debug(
							"Query execution time: "
								+ format.format(new Date(diff_time)));
					}

					qobj.close();
					ok = true;

				} catch (SQLException e) {
					stdlog_.error("Fatal SQLException in Dbase IO Thread", e);
				} catch (InterruptedException ie) {
					stdlog_.error(
						"Fatal InterruptedException in Dbase IO Thread",
						ie);
				} catch (RemoteException e) {
					stdlog_.error(
						"Fatal RemoteException in Dbase IO Thread",
						e);
				} catch (InvocationTargetException ite) {
					stdlog_.error(
						"Fatal InvocationTargetException in Dbase IO Thread",
						ite);
				} catch (MalformedURLException murle) {
				} catch (NotBoundException nb) {
				}

				if (!ok) {

					try {
						SwingUtilities.invokeAndWait(new Runnable() {
							public void run() {
								view_.queryFailed();
							}

						});
					} catch (InterruptedException e) {
					} catch (InvocationTargetException e) {
					}

				}

				synchronized (view_) {
					stdlog_.debug("Counter " + view_.counter_);
					if (decrementCounter() == 0) {
						view_.setThreadCompleted(true);
						break;
					}
				}
			}
		}

	}
	/**
			 * Method runQuery start the query from the engine. 
			 * Used by DBIOThread
			 * 			 * @return DbbQuery
			 * @throws RemoteException
			 * @throws SQLException
			 * @throws MalformedURLException
			 * @throws NotBoundException
			 */
	protected DbbQuery runQuery()
		throws
			RemoteException,
			SQLException,
			MalformedURLException,
			NotBoundException {
		return sqlEngine_.startQuery(getQueryChunks());
	}

	public void setVerifier(DbbVerifier verifier_) {
		DbbView.verifier_ = verifier_;
	}

	public static DbbVerifier getVerifier() {
		return verifier_;
	}

    /**
     * @param run_id void
     */
    public void hideColumn(String columnName) {
      getTable().hideColumn(columnName) ;
    }
    
	
    /**
     * 
     * @param selectedColumns
     * @return
     */
	public DbbPanel getColumnsPanelSubset(ArrayList selectedColumns){
		if (selectedColumns == null) throw new IllegalArgumentException(MsgManager.errPrecondition("selectedColumns attribute is null"));
		
		DbbPanel result = null;
		DbbWidget[] columns = columnsPanelFull_.getAllWidgets();
		
		result = new DbbPanel(columnsPanel_.getTitled().getTitle(), columnsPanel_.getColNumber());
		
		
		for (int i=0; i < selectedColumns.size(); i++){
			String lookingFor = (String) selectedColumns.get(i);
				boolean found = false;
				//Pay attention, weird for statement
				for (int j = 0;  found == false; j++){
					if (j < columns.length){
						String currentName = columns[j].getLabel().getText();
						if (lookingFor.equals(currentName)){
							//i found the DbbWidget i want!!!
							found = true;
							columns[j].reset();
							result.addWidget(columns[j]);
						}
					}else new ArrayIndexOutOfBoundsException("Cannot find the proper DbbWidget!!!");
				}	
			}
		
		for (int k = 0; k < columns.length; k++){
			if (columns[k].getLabel().getText().equals("RUN_ID")){
				result.addWidget(columns[k]);
			}
		}
		//now result DbbPanel should contains all the DbbWidget i want to display.
		return result;
	}
	
	public DbbPanel getRowsPanelSubset(ArrayList selectedRows){
		//Remember that the rowsPanel is called wherePanel!!
		assert (selectedRows != null): new IllegalArgumentException(MsgManager.errPrecondition("selectedRows attribute is null"));
		
		DbbPanel result = null;
		DbbWidget[] rows = rowsPanelFull_.getAllWidgets();
		
		result = new DbbPanel(wherePanel_.getTitled().getTitle(), wherePanel_.getColNumber());
		
		
		for (int i=0; i < selectedRows.size(); i++){
			String lookingFor = (String) selectedRows.get(i);
			boolean found = false;
			//Pay attention, weird for statement
			for (int j = 0;  found == false; j++){
				if (j < rows.length){
					String currentName = rows[j].getLabel().getText();
					if (lookingFor.equals(currentName)){
						//i found the DbbWidget i want!!!
						found = true;					
						rows[j].reset();
						result.addWidget(rows[j]);
					}
				}else new ArrayIndexOutOfBoundsException("Cannot find the proper DbbWidget!!!");
			}
		}
		//now result DbbPanel should contains all the DbbWidget i want to display.
		return result;
	}

	public DbbPanel getSortPanelSubset(ArrayList selectedSorting) {

		assert (selectedSorting != null): new IllegalArgumentException(MsgManager.errPrecondition("selectedRows attribute is null"));
	
		DbbPanel result = null;
		DbbWidget[] rows = sortPanelFull_.getAllWidgets();
		
		result = new DbbPanel(sortPanel_.getTitled().getTitle(), sortPanel_.getColNumber());
		
		
		for (int i=0; i < selectedSorting.size(); i++){
			String lookingFor = (String) selectedSorting.get(i);
			boolean found = false;
			//Pay attention, weird 'for' statement
			for (int j = 0;  found == false; j++){
				if (j < rows.length){
					String currentName = rows[j].getLabel().getText();
					if (lookingFor.equals(currentName)){
						//i found the DbbWidget i want!!!
						found = true;
						rows[j].reset();
						result.addWidget(rows[j]);
					}
				}else new ArrayIndexOutOfBoundsException("Cannot find the proper DbbWidget!!!");
			}		
		}
		//now result DbbPanel should contains all the DbbWidget i want to display.
		return result;
	}

	public DbbButtonPanel getSummaryButtons_() {
		return summaryButtons_;
	}

	public boolean isStartQueryOnEnter() {
		return isStartQueryOnEnter;
	}
	

	public void setStartQueryOnEnter(boolean isStartQueryOnEnter) {
		this.isStartQueryOnEnter = isStartQueryOnEnter;
	}

	public DbbPanel getColumnsPanelFull() {
		return columnsPanelFull_;
	}
	

	public DbbPanel getRowsPanelFull() {
		return rowsPanelFull_;
	}
	

	public DbbPanel getSortPanelFull() {
		return sortPanelFull_;
	}
}
