package org.eso.ohs.core.dbb.client;

import java.awt.Component;
import java.awt.event.ActionEvent;

import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JOptionPane;
import javax.swing.JTextField;

import org.eso.ohs.core.gui.baseaction.ActionSuperclass;
import org.eso.ohs.core.gui.widgets.SimpleInputDialog;

/**
 * The goal to this action is to save and
 * activate from menu
 * a widget configuration to the
 * DbbPanels columns, rows, sort.<p>
 * This action gets the current widget selection and
 * adds to a menu a new submenu.
 * The selection of this submenus retrieves and
 * applies the saved widget selection to the
 * DbbPanels columns, rows, sort.
 * The setting is saved in a file.
 * OHS Keywords used:readOnly, defaultMenuFolder,defaultMenuName,
 * rows,cols,sort, menuRoot, menuName
 * <br>
 * @version $Revision: 1.8 $
 * @author D. Dorigo
 * @author cmavros 24May02 Rearranged code to make methods more accessable to
 *          to the DefaultConfigAction().
 */
public class ManageConfigAction extends ActionSuperclass {

    public static final String cvsID_ =
        "$Id: ManageConfigAction.java,v 1.8 2005/09/09 15:04:00 gdonino Exp $";

    /**
     * An OHSConfig default menu setting child: if "" the menu name is
     * in the main menu.
     */
  

    private JFrame parentFrame_;
    private boolean singleMenu_ = false;
  //  private MenuMap menuMap_ = null;
    private String requestDefault_;
    private final String DEFAULT = "default";
    private final String DEFAULT_REQUEST = "Save As Default";
    private DbbViewConfigurationManager viewConfManager_;

    /**
     * @param parentFrame the frame
     * @param menu the menu in which config is added
     * @param rowPanel The DbbWiew filter panel
     * @param colPanel The DbbWiew show panel
     * @param sortPanel The DbbWiew sorting panel
     * @param menuRoot The menu path
     * @param helperConfFile the configuration file
     * @param name the action name is used as menu item name
     * @param singleMenu foldering of menu actions
     */
    public ManageConfigAction(
        JFrame parentFrame,
        JMenu menu,
        String menuRoot,
        DbbViewConfigurationManager viewConfManager,
        String name,
        boolean singleMenu) {

        super(parentFrame, name);
  
        parentFrame_ = parentFrame;
        singleMenu_ = singleMenu;
        requestDefault_ = name;
        viewConfManager_ = viewConfManager;
    }


    /**
     * Action starting method
     * @param ev       user-initiated ActionEvent.
     */
    public void actionPerformedImpl(ActionEvent ev) {
        String menuName = null;
        if (requestDefault_.equals(DEFAULT_REQUEST)) {
            menuName = DEFAULT;
        } else {
            menuName = getMenuName();
        }
        if (menuName != null) {
            boolean addedMenu = viewConfManager_.addMenuToOHSConfig(menuName);
            if (addedMenu) {
                viewConfManager_.saveConfig();
            }
        }
    }

    /**
     * Request menu from user.
     * If the nothing is inputted return null.
     * @return String The configuration menu name to be added
     */
    private String getMenuName() {
        String menuName = null;
        if (!singleMenu_)
            menuName = getMenuViaDialog();
        else
            menuName = getSingleMenuViaDialog();

        // if not cancel op but menu empty
        if (menuName != null && menuName.length() == 0) {
            JOptionPane.showMessageDialog(
                parentFrame_,
                "Cannot save the setting: the menu name is empty.");
            menuName = null;
        }
        return menuName;
    }

    /**
     * Starts an menu dialog in which the client inserts the menu name
     */
    //  pay attention, this method can return null value
    private String getSingleMenuViaDialog() {
    	String result = null;
        JTextField newNameTF = new JTextField();

        SimpleInputDialog dlg =
            new SimpleInputDialog(
                parentFrame_,
                "Save Settings",
                "Enter details:\n",
                new String[] { "Settings Name:" },
                new Component[] { newNameTF });
        dlg.show();

        if (dlg.getUserChoice() == JOptionPane.OK_OPTION) {
            result = newNameTF.getText();
        } 
        
        return result;
    }

    /**
     * Starts an menu dialog in which the client inserts the menu name
     */
    //pay attention, this method can return null value
    private String getMenuViaDialog() {
    	String result = null;
      //  menuFolderKeyValue_ = EMPTY_NAME;
        JTextField newNameTF = new JTextField();
        JTextField newRootNameTF = new JTextField();
        SimpleInputDialog dlg =
            new SimpleInputDialog(
                parentFrame_,
                "Save Settings",
                "Enter details:\n",
                new String[] { "Folder Name <optional>:", "Settings Name:" },
                new Component[] { newRootNameTF, newNameTF });
        dlg.show();
        if (dlg.getUserChoice() == JOptionPane.OK_OPTION) {
            viewConfManager_.setMenuFolderKeyValue(newRootNameTF.getText());
            result = newNameTF.getText();
        } 
        
        return result;
    }
}

