package org.eso.ohs.core.dbb.sql;

// $Id: DbbSqlQueryChunks.java,v 1.2 2004/04/08 12:09:51 tcanavan Exp $

import java.io.Serializable;

import org.apache.log4j.Logger;

/**
 * This class defines a data structure containing all the sql chunks
 * needed to define a sql query and passed from the client (DbbView)
 * to the server (DbbSqlEngine) as a data unit.
 *
 * <br>
 * @see DbbSqlChunk
 * @author Carlo Boarotto, ESO 20-Jul-1999 
 */
public class DbbSqlQueryChunks implements Serializable {

private static final String rcsid = "$Id: DbbSqlQueryChunks.java,v 1.2 2004/04/08 12:09:51 tcanavan Exp $";
private static Logger stdlog_ = 
    Logger.getLogger( DbbSqlQueryChunks.class );
 
    /** Sql chunks defining the requested columns */
    public DbbSqlChunk[] columns = null;

    /** Sql chunks defining sql clauses in the where statement */
    public DbbSqlChunk[] rows = null;

    /** Sql chunks defining the requested sorting criteria */
    public DbbSqlChunk[] sort = null;

    /** Sql chunks defining the requested grouping criteria */
    public DbbSqlChunk[] group = null;


    //----------------------------------------
    // Public methods
    //----------------------------------------

    /**
     * Append new sql clauses to the already defined within the
     * current instance.
     *
     * @param new_rows New chunks to be taken into account 
     */
    public void appendRows(DbbSqlChunk[] new_rows) {

	if(new_rows == null)
	    return;

	if(rows == null) {
	    rows = new_rows;
	    return;
	}

	int len1 = rows.length;
	int len2 = new_rows.length;

	DbbSqlChunk[] tmp_array = new DbbSqlChunk[len1 + len2];

	System.arraycopy(rows,
			 0,
			 tmp_array,
			 0,
			 len1);
	// printArray(tmp_array);
	System.arraycopy(new_rows,
			 0,
			 tmp_array,
			 len1,
			 len2);
	// printArray(tmp_array);

	rows = tmp_array;
    }

    //----------------------------------------
    // Debug methods
    //----------------------------------------

    /** Print an array of sql chunks */
    private static void printArray(DbbSqlChunk[] a) {
	StringBuffer printOut = new StringBuffer();
	printOut.append("Array " + a + " : ");
	for(int i=0; i<a.length; i++)
	    printOut.append("<" + a[i] + "> ");
	stdlog_.debug(printOut.toString());
    }
}

