package org.eso.ohs.core.dbb.xml;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.Vector;

import org.apache.log4j.Logger;
import org.eso.ohs.core.utilities.ConnectionPool;
import org.eso.ohs.core.utilities.ObjectIOException;

/**
 * retrieves data from DB hiding the connection operations.
 * <p>
 * $Id: DbbXmlDataHelper.java,v 1.4 2005/11/04 10:55:29 ddorigo Exp $
 *
 * <br>
 * @author D. Dorigo
 *
 */
public class DbbXmlDataHelper {

	/**
	 * Field url
	 */
	private String url = null;
	/**
	 * Field userName
	 */
	private String userName = null;
	/**
	 * Field passwd
	 */
	private String passwd = null;
	/**
	 * Field stdlog_
	 */
	private static Logger stdlog_ =
		Logger.getLogger(DbbXmlDataHelper.class);

	/**
	 * Field BUFFERSIZE
	 */
	private static final int BUFFERSIZE = (1024 * 1024);
	
	/**
	 * Field singleton
	 */
	private static DbbXmlDataHelper singleton = null;

	/** Does nothing */
	public DbbXmlDataHelper() {
	}

	/** 
	 * 
	 * @param url String
	 * @param userName String
	 * @param passwd String
	 */
	public DbbXmlDataHelper(String url, String userName, String passwd) {
		initializeConnection(url, userName, passwd);
	}

	/**
	 * Method initializeConnection variables. 
	 * By default they are taken from Config.getCfg()
	 * @param url String
	 * @param userName String
	 * @param passwd String
	 */
	public void initializeConnection(
		String url,
		String userName,
		String passwd) {
		this.url = url;
		this.userName = userName;
		this.passwd = passwd;
	}

	/**
	 * Method getInstance
	 * @return DBdataHelper
	 */
	public static DbbXmlDataHelper getInstance() {
		if (singleton == null) {
			singleton = new DbbXmlDataHelper();
		}
		return singleton;
	}

	/**
	 * Returns the cached JDBC database Connection object creating
	 * it first, if necessary. It gets the connection info from Config
	 * class.
	 *
	 * @return  JDBC Connection object
	 *
	 * @exception ObjectIOException
	 *		    an error occured during an access to a business object
	 * @exception SQLException
	 *		    an error occurred during the dbase operation
	 */
	public Connection acquireConnection()
		throws ObjectIOException, SQLException {
		if (url == null || userName == null || passwd == null) {
			//Config cfg = Config.getCfg();
			//url = cfg.getUrl("OR");
			//userName = cfg.getDbUserName();
			//passwd = cfg.getDbPassword();
			//TODO - use Config as above
			url = "jdbc:sybase:Tds:acdbdev.hq.eso.org:6789";
			userName = "opc";
			passwd = "andromeda";
		}

		return ConnectionPool.getPool().acquireConnection(
			url,
			userName,
			passwd);
	}

	/**
	 * Returns an array of String from the select clause.
	 * If data are not found, a 0 length array is returned.
	 *
	 * @param selectClause	retriever of a list of objects.
	 *
	 * @return the array of objects
	 *
	 * @exception ObjectIOException
	 *			If sql error.
	 */
	public String[] readStringArray(String selectClause)
		throws ObjectIOException {

		String[] usoArray = null;
		Connection con = null;
		Statement stmt = null;

		ResultSet rs = null;

		try {
			// initialize the connection
			con = acquireConnection();

			stmt = con.createStatement();
			
			return readStringArray(selectClause, stmt);
			
		} catch (SQLException e) {
			stdlog_.error("Sql statement error: " + selectClause, e);
			throw new ObjectIOException(e);
		} catch (IllegalArgumentException e) {
			stdlog_.error(
				"IllegalArgumentException error Sql statement: " + selectClause,
				e);
			throw new ObjectIOException(e);
		} finally {
			freeConnection(con, stmt);
		}
	}
	

	/**
	 * Method readStringArray
	 * @param selectClause String
	 * @param stmt Statement
	 * @return String[]
	 * @throws SQLException
	 */
	 
	public String[] readStringArray(String selectClause, Statement stmt)
		throws SQLException {
		String[] usoArray;
		ResultSet rs;
		int fieldId = 1;
		Vector result = new Vector();
		// execute query
		rs = execQuery(selectClause, stmt);
		// get data
		while (rs.next()) {
			result.addElement(rs.getString(fieldId));
		}
		rs.close();
		usoArray = new String[result.size()];
		result.copyInto(usoArray);
		return usoArray;
	}
	
	/**
	 * Method execQuery
	 * @param selectClause String
	 * @param stmt Statement
	 * @return ResultSet
	 * @throws SQLException
	 */
	public ResultSet execQuery(String selectClause, Statement stmt)
		throws SQLException {
		ResultSet rs;
		stdlog_.debug(selectClause);
		rs = stmt.executeQuery(selectClause);
		return rs;
	}

	// releases the connection.
	/**
	 * Method freeConnection
	 * @param con Connection
	 * @param stmt Statement
	 * @throws ObjectIOException
	 */

	public void freeConnection(Connection con, Statement stmt) {
		stdlog_.debug(" freeConnection");
		if (con != null) {
			try {
				if (stmt != null) {
					stmt.close();
				}
			} catch (SQLException e) {
				stdlog_.error("SQLException ", e);
			} finally {
				try {
					releaseConnection(con);
				} catch (ObjectIOException e1) {
					stdlog_.error("Error while releasing connection: ", e1);
				}
			}
		}
	}
	/**
	 * Method freeConnection
	 * @param con Connection
	 * @param stmt Statement
	 * @throws ObjectIOException
	 */

	private void freeConnection(Connection con) {
		stdlog_.debug(" freeConnection");
		if (con != null) {
			try {
				releaseConnection(con);
			} catch (ObjectIOException e1) {
				stdlog_.error("Error while releasing connection: ", e1);
			}
		}
	}
	/**
	* Method setAutoCommitAndFreeConnection
	* set autocommit to true and release the connection
	* param con Connection
	* * @param con Connection
	@param stmt Statement
	* @throws ObjectIOException
			*/
	public void setAutoCommitAndFreeConnection(Connection con, Statement stmt)
		throws ObjectIOException {
		if (con != null) {
			try {
				con.setAutoCommit(true);
			} catch (SQLException e) {
				stdlog_.error(e);
			}
		}
		freeConnection(con, stmt);
	}

	/**
	 * Method rollbackConnection
	 * @param conn Connection
	 */
	public void rollbackConnection(Connection conn) {

		try {
			if (conn != null) {
				conn.rollback();
			}
		} catch (SQLException e1) {
			stdlog_.error("rollback failed: ", e1);
		}
	}

	/**
	 * Returns an integer of a select. It should be one row only (e.g count(), max...)
	 *
	 * @param selectClause	retriever of a list of objects.
	 *
	 * @return integer
	 *
	 * @exception ObjectIOException
	 *			sql exception
	 */
	public String readString(String selectClause) throws ObjectIOException {

		Connection con = null;
		Statement stmt = null;
		String result = "";

		try {

			// initialize the connection
			con = acquireConnection();
			stmt = con.createStatement();
			// execute query
			result = readString(selectClause, stmt);
			return result;
		} catch (SQLException e) {
			stdlog_.error("Sql statement error: " + selectClause, e);
			throw new ObjectIOException(e);
		} catch (IllegalArgumentException e) {
			stdlog_.error("IllegalArgumentException error", e);
			throw new ObjectIOException(e);
		} finally {
			freeConnection(con, stmt);
		}
	}

	/**
	 * Method readIntValue
	 * @param selectClause String
	 * @param stmt Statement
	 * @return int
	 * @throws SQLException
	 * @throws ObjectIOException
	 */
	public String readString(String selectClause, Statement stmt)
		throws SQLException, ObjectIOException {
		int fieldId = 1;
		ResultSet rs = execQuery(selectClause, stmt);
		// get data
		String result;
		if (rs.next()) {
			result = rs.getString(fieldId);
			rs.close();
		} else {
			rs.close();
			throw new ObjectIOException(
				" value not found in select: " + selectClause);
		}

		return result;
	}

	/**
	 * Returns the cached JDBC database Connection object creating
	 * it first, if necessary.
	 *
	 * @param con Connection
	 * @exception ObjectIOException
	 *		    an error occured during an access to a business object
	 */
	public void releaseConnection(Connection con) throws ObjectIOException {
		ConnectionPool.getPool().releaseConnection(con);
	}

	/**
	 * Updates, deletes or inserts data into the DB.
	 *
	 * @param 	updateClause the delete, insert or update statement.
	 *
	 * @exception ObjectIOException
	 *			sql error.
	 */
	public void execUpdate(String updateClause) throws ObjectIOException {

		Connection con = null;
		Statement stmt = null;
		try {
			// initialize the connection
			con = acquireConnection();
			stmt = con.createStatement();
			// execute query
			execUpdate(updateClause, stmt);
		} catch (SQLException e) {
			e.printStackTrace();
			stdlog_.error(e);
			throw new ObjectIOException(e);
		} catch (IllegalArgumentException e) {
			stdlog_.error(e);
			throw new ObjectIOException(e);
		} finally {
			freeConnection(con, stmt);
		}
	}

	/**
	 * Method execUpdate
	 * @param updateClause String
	 * @param stmt Statement
	 * @throws SQLException
	 */
	public void execUpdate(String updateClause, Statement stmt)
		throws SQLException {
		stdlog_.debug(updateClause);
		stmt.executeUpdate(updateClause);
	}

}
