package org.eso.ohs.core.docview.datatrans;

import java.beans.PropertyChangeListener;

/**
 * Interface defining the minimal set of methods that a Java object
 * must implement in order to be linked to a GUI component using
 * the DocView collection of classes.
 * 
 * <p>
 * Of course, it is implicitly assumed that the object will also have
 * a method to fire the change events when a setter method is called.
 * That, however, is private to the object concerned.
 * 
 * <p>
 * $Id: DocViewObject.java,v 1.2 2004/03/16 10:51:41 tcanavan Exp $
 * <br>
 * @author Bob Kemp
 * 
 * @see org.eso.ohs.core.docview.gui.DocView
 * @see javax.swing.event.EventListenerList
 */
public interface DocViewObject {

    /**
     * Add a PropertyChangeEvent listener for the object.
     * 
     * @param lnr	a PropertyChangeEVent listener
     */
    public void addPropertyChangeListener(PropertyChangeListener lnr);

    /**
     * Remove a PropertyChangeEvent listener for the object.
     * 
     * @param lnr	a PropertyChangeEVent listener
     */
    public void removePropertyChangeListener(PropertyChangeListener lnr);

}

