package org.eso.ohs.core.docview.gui;

import java.awt.Color;

import javax.swing.JSlider;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;


/**
 * A GUI adaptor to interface between DocView and Swing JSlider objects.
 * By using a GuiAdaptor, DocView itself can work with a much wider range
 * of GUI components.
 * 
 * <p>
 * $Id: SliderAdaptor.java,v 1.1 2004/02/17 12:38:00 tcanavan Exp $
 * <p>
 * @author Bob Kemp
 */
public class SliderAdaptor implements GuiAdaptor, ChangeListener {

    /** GUI component whose value is to be read/updated */
    private JSlider slider_;

    /** Has the GUI component been modified?  Reset by DocView. */
    private boolean modified_;

    /**
     * Constructor
     * 
     * @param slider	GUI component to be linked to
     */
    public SliderAdaptor(JSlider slider) {
	slider_ = slider;
	slider.addChangeListener(this);
    }


    /**
     * Set the background colour of the underlying GUI component
     * 
     * @param bg	new value of background colour
     */
    public void setForeground(Color bg) {
	slider_.setForeground(bg);
    }


    /**
     * Enable the underlying GUI component
     * 
     * @param enabled	true if component should be enabled, false otherwise
     */
    public void setEnabled(boolean enabled) {
	slider_.setEnabled(enabled);
    }

    /**
     * Is the underlying GUI component enabled?
     * 
     * @return	true if component should be enabled, false otherwise
     */
    public boolean isEnabled() {
	return slider_.isEnabled();
    }

    /**
     * Set a new value for the GUI components value
     * 
     * @param newValue	new value represented as a String
     */
    public void setValue(String newValue) {
	if (newValue != null) {
	    slider_.setValue(new Integer(newValue).intValue());
	}
    }

    /**
     * Get the current value of the GUI component
     * 
     * @return	current value represented as a String
     */
    public String getValue() {
	return String.valueOf(slider_.getValue());
    }

    /**
     * Flag the component as having been modified (or not modified)
     * 
     * @param	new value of the flag
     */
    public void setModified(boolean modified) {
	modified_ = modified;
    }

    /**
     * Has the GUI component been modified?
     * 
     * @return	current value of the flag
     */
    public boolean isModified() {
	return modified_;
    }


    /** A listener for changes to the underlying GUI component */
    public void stateChanged(ChangeEvent ev) {
	 setModified(true);
    }

}

