package org.eso.ohs.core.gui.models;

import javax.swing.text.AttributeSet;
import javax.swing.text.BadLocationException;
import javax.swing.text.PlainDocument;

/**
 * A Document model for JTextField (and JTextArea) instances that
 * only allows the insertion of a specific subset of characters.
 * 
 * <p>
 * $Id: BoNameModel.java,v 1.1 2004/02/17 10:35:21 tcanavan Exp $
 */
public class BoNameModel extends PlainDocument {
    /** Characters that the user is allowed to insert into a Name field */
    public static final String allowedCharacters
	= "abcdefghijklmnopqrstuvwxyz"
	+ "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
	+ "0123456789"
	+ "+-_ .";

    private int maxLen_ = 0;

    /**
     * Constructor that gets maxLenth of text to accept from ModelCfg.
     */
    public BoNameModel() {
        maxLen_ = ModelsCfg.getCfg().getMaxNameLength();
    }

    /**
     * Inserts legal characters from the given string into this Document.
     * 
     * @param offset	offset in the text buffer at which to insert string.
     * @param str	string to be inserted.
     * @param attr	text attributes.
     * 
     * @exception BadLocationException	an illegal offset was specified.
     */
    public void insertString(int offset, String str, AttributeSet attr)
		throws BadLocationException {
	int len = getLength();
	if ((maxLen_ >= 0) && (len+str.length() > maxLen_)) {
			// If insertion would make text too long, just return.
	    return;
	}

	for (int j=str.length()-1; j >= 0; j--) {
	    char ch = str.charAt(j);
	    if (allowedCharacters.indexOf(ch) < 0) {
		    // Delete the illegal character and leave the rest.
		str = str.substring(0, j) + str.substring(j+1);
	    }
	}

	super.insertString(offset, str, attr);
    }

}

