package org.eso.ohs.core.gui.widgets;

import java.awt.Dimension;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.GridLayout;
import java.awt.Toolkit;

import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JWindow;
import javax.swing.SwingConstants;

import org.eso.ohs.core.gui.utilities.ImageUtils;

/**
 * Splash screen to display at startup.
 * 
 * <p>
 * $Id: SplashScreen.java,v 1.4 2005/10/11 15:00:06 ddorigo Exp $
 */

public class SplashScreen extends JWindow {

    private SplashPanel content_;

    public SplashScreen() {
	this("Loading...");
    }

    public SplashScreen(String message, String image) {
	content_ = new SplashPanel();
	content_.setMessage(message);
	content_.setLayout(new GridLayout(1,2));

	setContentPane(content_);

        JLabel splashLabel = new JLabel();

        splashLabel.setIcon(
    	    ImageUtils.getIcon(image)
        );
        splashLabel.setHorizontalTextPosition(SwingConstants.CENTER);
        splashLabel.setVerticalTextPosition(SwingConstants.BOTTOM);
        content_.add(splashLabel);

        pack();
	Dimension splashSize = getPreferredSize();
	Dimension screen = Toolkit.getDefaultToolkit().getScreenSize();

	setLocation(
            (screen.width-splashSize.width)/2,
            (screen.height-splashSize.height)/2
	);

    }

    public SplashScreen(String message) {
	content_ = new SplashPanel();
	content_.setMessage(message);
	content_.setLayout(new GridLayout(1,2));

	setContentPane(content_);

	for (int j=1; j <= 2; j++) {
	    JLabel splashLabel = new JLabel();

	    splashLabel.setIcon(
		ImageUtils.getIcon("splash.gif")
	    );
	    splashLabel.setHorizontalTextPosition(SwingConstants.CENTER);
	    splashLabel.setVerticalTextPosition(SwingConstants.BOTTOM);
	    content_.add(splashLabel);
	}

	pack();
	Dimension splashSize = getPreferredSize();
	Dimension screen = Toolkit.getDefaultToolkit().getScreenSize();

	setLocation(
	    (screen.width-splashSize.width)/2,
	    (screen.height-splashSize.height)/2
	);

    }

    public void setMessage(String message) {
	content_.setMessage(message);
    }

    public String getMessage() {
	return content_.getMessage();
    }

    private static class SplashPanel extends JPanel {
	private String message_ = "Loading...";

	private boolean messageChanged_ = true;
	private int x_, y_;

	public void setMessage(String msg) {
	    message_ = msg;
	    messageChanged_ = true;
	}

	public String getMessage() {
	    return message_;
	}

	private void positionMessage(Graphics g) {
	    messageChanged_ = false;

	    Dimension winSize = getSize();
	    FontMetrics metrics = g.getFontMetrics();
	    int strWidth = metrics.stringWidth(message_);
	    int strHeight = metrics.getHeight();

	    x_ = (winSize.width-strWidth)/2;
	    y_ = (winSize.height-strHeight)/2;

	    if (x_ < 0) x_ = 0;
	    if (y_ < 0) y_ = 0;

	}

	public void paint(Graphics g) {
	    super.paint(g);	// paint background image

	    if (message_ != null) {
		if (messageChanged_) {
		    positionMessage(g);
		}

		g.drawString(message_, x_, y_);
	    }
	}
    }

    public static void main(String[] args) {
	// A M Chavan, 21-Nov-2000: added args[0] to constructor
	SplashScreen wdw = new SplashScreen( args[0] );
	wdw.setVisible(true);

    }

}

