package org.eso.ohs.core.utilities;

import java.util.EventObject;

/**
 * Encapsulates information describing changes to the system state.
 *
 * <p>
 * $Id: SystemStatusEvent.java,v 1.1 2004/02/17 14:31:55 tcanavan Exp $
 * <br>
 * @author Bob Kemp
 */

public class SystemStatusEvent extends EventObject {

    /**
     * Constant to identify a shutdown event.
     */
    public static final int SHUTDOWN = 1;

    /**
     * Constant to identify a re-login event.
     */
    public static final int INITIALISATION = 2;

    /**
     * Constant to identify a re-login event.
     */
    public static final int RELOGIN = 3;

    /**
     * Constant to identify a synchronize event.
     * Upon receipt of this a component should (eg) stop editing
     * of any field and, if appropriate, save the result.
     */
    public static final int SYNC = 5;

   /**
    * Contains a value of one of the constants defined above
    */
    private int type_;

    /**
     * A reference to the object causing the event.
     */
    private Object obj_;


    /**
     * Constructor for SystemStatusEvent.
     * 
     * @param type  	type of event: SHUTDOWN, RELOGIN, SYNC, etc
     * @param obj	the object that has caused the event.
     */
    public SystemStatusEvent(int type, Object obj) {
	super(obj);
	type_       = type;
	obj_	    = obj;
    }

    /**
     * Retrieves the type of SystemStatusEvent. 
     * 
     * @return Type of event
     */
    public int getType() {
	return type_;
    }

    /**
     * Retrieves the object associated with the SystemStatusEvent.
     *
     * @return Reference to the object that the event relates to.
     */
    public Object getObject() {
	return obj_;
    }

}

