/* $Id: DbaseHandlerPhase1Target.java,v 1.2 2005/04/06 16:38:12 ddorigo Exp $ */

package org.eso.ohs.persistence.dbase.phase1;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.LinkedHashSet;
import java.util.Set;
import java.util.Vector;

import org.apache.log4j.Logger;
import org.eso.ohs.core.utilities.ObjectIOException;
import org.eso.ohs.core.utilities.TextUtils;
import org.eso.ohs.dfs.BusinessVisitor;
import org.eso.ohs.dfs.Phase1Target;
import org.eso.ohs.dfs.StorableObject;
import org.eso.ohs.persistence.Config;
import org.eso.ohs.persistence.ObjectNotFoundException;
import org.eso.ohs.persistence.StorageManager.UnpackedStorableObject;
import org.eso.ohs.persistence.dbase.DbaseIOHandler;

/**
 * Sql handler to retrieve, to get and to updateEsoUser objects
 * on the database
 * <br>
 * $Id: DbaseHandlerPhase1Target.java,v 1.2 2005/04/06 16:38:12 ddorigo Exp $
 * @author Carlo Boarotto, ESO 21-Mar-2000
 */
public class DbaseHandlerPhase1Target extends DbaseIOHandler {

	private static long MAX_PHASE1_TARGET = 10000;

	/**
	 * Field stdlog_
	 */
	private static Logger stdlog_ =
		Logger.getLogger(DbaseHandlerPhase1Target.class);

	/**
	 * Managed class
	 * @return EsoUser class
	 */
	public Class getManagedClass() {
		return Phase1Target.class;
	}

	/** A reference to the singleton DbaseHandlerEsoUser instance. */
	private static DbaseHandlerPhase1Target singleton_ = null;

	/**  column names */
	private static String PROG_ID = "programme_id";

	public static final String ID = "id";
	/**
	 * Field ITEM_NAME
	 */
	public static final String ITEM_NAME = "item_name";
	/**
	 * Field SURNAME
	 */
	public static final String R_A = "r_a";
	/**
	 * Field SURNAME_DL
	 */
	public static final String DEC = "dec";
	/**
	 * Field PASSWD
	 */
	public static final String MAGNITUDE = "magnitude";

	/** eso_users obsolete (and not null) column names */
	public static final String DIAMETER = "angular_diam";
	/**
	 * Field EMAIL
	 */
	public static final String COMMENT = "comments";
	/**
	 * Field PANEL
	 */
	public static final String EQUINOX = "equinox";
	/**
	 * Field time requested  integer: (hours * 10)
	 */
	public static final String TIME_ON_TARGET = "time_targets";
	/**
	 * Field CHAIR
	 */
	public static final String RUN_REFERENCES = "run_references";

	/** reference_stars column names */
	public static final String REF_STARS_NAME = "item_name";
	public static final String REF_STARS_TARGET_ID = "target_id";
	private static String REF_STARS_PROG_ID = "programme_id";

	/** Sql select statement to retireve EsoUser entry. other fileds could be added in subclass */
	protected static final String target_fields =
		Phase1SelectStmt.targetsTable.get()
			+ "."
			+ ID
			+ ","
			+ Phase1SelectStmt.targetsTable.get()
			+ "."
			+ PROG_ID
			+ ","
			+ Phase1SelectStmt.targetsTable.get()
			+ "."
			+ ITEM_NAME
			+ ","
			+ Phase1SelectStmt.targetsTable.get()
			+ "."
			+ R_A
			+ ","
			+ Phase1SelectStmt.targetsTable.get()
			+ "."
			+ DEC
			+ ","
			+ Phase1SelectStmt.targetsTable.get()
			+ "."
			+ TIME_ON_TARGET
			+ ","
			+ Phase1SelectStmt.targetsTable.get()
			+ "."
			+ MAGNITUDE
			+ ","
			+ Phase1SelectStmt.targetsTable.get()
			+ "."
			+ DIAMETER
			+ ","
			+ Phase1SelectStmt.targetsTable.get()
			+ "."
			+ COMMENT
			+ ",\n"
			+ Phase1SelectStmt.targetsTable.get()
			+ "."
			+ EQUINOX
			+ ","
			+ Phase1SelectStmt.targetsTable.get()
			+ "."
			+ RUN_REFERENCES;

	protected static final String star_fields =
		Phase1SelectStmt.targetsTable.get()
			+ "."
			+ REF_STARS_TARGET_ID
			+ ","
			+ Phase1SelectStmt.targetsTable.get()
			+ "."
			+ REF_STARS_PROG_ID
			+ ","
			+ Phase1SelectStmt.targetsTable.get()
			+ "."
			+ REF_STARS_NAME;
	protected final String target_select_sql =
		"SELECT DISTINCT " + target_fields;

	protected final String target_sql =
		target_select_sql
			+ "\n FROM "
			+ Phase1SelectStmt.targetsTable.get()
			+ " ";

	public static int targetIdFromId(long id) {
		return (int) (id % MAX_PHASE1_TARGET);
	}

	public static long programmeIdFromId(long id) {
		return id / MAX_PHASE1_TARGET;
	}

	public static long idForPhase1Target(long targetId, long progId) {
		return progId * MAX_PHASE1_TARGET + targetId;
	}

	/** ctor */
	public DbaseHandlerPhase1Target() {
		super();
	}

	/**
	  * Read user data
	  * @param stmt statement
	 * @param objId id to be read
	 * @return UnpackedStorableObject
	 * @exception ObjectIOException IO Eror occured while reading from the database
	  * @exception ObjectNotFoundException the requested object was not found in the database
	  * @exception SQLException an error occurred during the dbase operation
	  */
	public UnpackedStorableObject read(Statement stmt, long objId)
		throws ObjectIOException, ObjectNotFoundException, SQLException {

		Phase1Target target = (Phase1Target) newInstance();
		UnpackedStorableObject uso = new UnpackedStorableObject(target);

		String sql =
			target_sql
				+ " WHERE "
				+ ID
				+ " = "
				+ targetIdFromId(objId)
				+ " AND "
				+ PROG_ID
				+ " = "
				+ programmeIdFromId(objId);
		stdlog_.debug(sql);
		ResultSet rs = stmt.executeQuery(sql);

		if (!rs.next()) { // Only one with this id !!
			throw new ObjectNotFoundException("Object not found in database");
		}

		target.setId(
			Config.getCfg().tableToUniqueId(objId, Phase1Target.class));
		setTargetFields(target, rs);
		rs.close();
		// load the affiliation for the user
		loadReferenceStars(stmt, target);
		return new UnpackedStorableObject(target);
	}

	/**
	 * Read data objects are unique)
	 * @param stmt statement
	 * @param whereClause where condition for retreiving the object
	 * @param otherTables String table condition
	 * @return UnpackedStorableObject[]
	 * @exception ObjectIOException IO Eror occured while reading from the database
	 * @exception ObjectNotFoundException the requested object was not found in the database
	 * @exception SQLException an error occurred during the dbase operation
	 */
	public UnpackedStorableObject[] read(
		Statement stmt,
		String whereClause,
		String otherTables)
		throws ObjectIOException, ObjectNotFoundException, SQLException {

		if (otherTables == null) {
			otherTables = "";
		}

		UnpackedStorableObject[] target_array = null;
		Phase1Target target = null;
		String sql = target_sql + otherTables + "\n WHERE " + whereClause;
		stdlog_.debug(sql);
		ResultSet rs = stmt.executeQuery(sql);

		try {
			Set targets = new LinkedHashSet();
			while (rs.next()) {
				target = (Phase1Target) newInstance();

				long targetId = rs.getInt(ID);
				long progId = rs.getInt(PROG_ID);
				// Set only when attaching the object as a new
				// element
				target.setId(
					Config.getCfg().tableToUniqueId(
						idForPhase1Target(targetId, progId),
						Phase1Target.class));
				setTargetFields(target, rs);
				// New element
				targets.add(target);
			}
			rs.close();

			Phase1Target[] targetList =
				(Phase1Target[]) targets.toArray(
					new Phase1Target[targets.size()]);
			target_array = new UnpackedStorableObject[targetList.length];
			// add the affiliation to the user and insert them in array
			for (int i = 0; i < target_array.length; i++) {
				target = targetList[i];
				loadReferenceStars(stmt, target);
				target_array[i] = new UnpackedStorableObject(target);
			}
		} catch (SQLException e) {
			stdlog_.error(e);
			throw new ObjectIOException(e);
		}
		return target_array;
	}

	/**
	 * Set the user fields from the result set.
	 * Only for avoiding duplicated code
	 *
	 * @param user EsoUser
	 * @param rs ResultSet
	 * @throws SQLException
	 */
	protected void setTargetFields(Phase1Target target, ResultSet rs)
		throws SQLException {
		target.setTargetId(rs.getInt(ID));
		target.setProgId(rs.getInt(PROG_ID));
		target.setItemName(rs.getString(ITEM_NAME));
		target.setRa(rs.getInt(R_A));
		target.setDeclination(rs.getInt(DEC));
		target.setTimeOnTarget(rs.getInt(TIME_ON_TARGET));
		target.setMagnitude(rs.getString(MAGNITUDE));
		target.setDiameter(rs.getString(DIAMETER));
		target.setUserComment(rs.getString(COMMENT));
		target.setEquinox(rs.getDouble(EQUINOX));
		target.setMagnitude(rs.getString(MAGNITUDE));
	}

	/**
	  * Read readAffiliations data
	  * @param stmt statement
	 * @param user id to be read
	  * @param userId long
	 * @exception ObjectIOException IO Eror occured while reading from the database
	  * @exception ObjectNotFoundException the requested object was not found in the database
	  * @exception SQLException an error occurred during the dbase operation
	  */
	protected void loadReferenceStars(Statement stmt, Phase1Target target)
		throws ObjectIOException, ObjectNotFoundException, SQLException {

		Vector refStarList = new Vector();
		String star_sql =
			star_fields
				+ " WHERE "
				+ REF_STARS_TARGET_ID
				+ " = "
				+ target.getTargetId()
				+ " AND "
				+ REF_STARS_PROG_ID
				+ " = "
				+ target.getProgId();
		stdlog_.debug(star_sql);
		ResultSet rs = stmt.executeQuery(star_sql);

		// get the affiliation list reusing the statement
		if (!rs.next()) { // Only one with this id !!
			throw new ObjectNotFoundException("Object not found in database");
		}

		String refStar = rs.getString(REF_STARS_NAME);
		target.setReferenceStar(refStar);
		rs.close();
	}

	/**
	 * Throws an ObjectIOException -- the phase 1 database cannot be
	 * changed from within P2PP.
	 *
	 * @param stmt  JDBC Statement for contacting the dbase
	 * @param id    per-table id of the object to be written.
	 * @param obj   an array: the first element is the BO to be stored.
	 *
	 * @exception ObjectIOException     operation is not permitted exception
	 *
	 * @see org.eso.ohs.persistence.ObjectManager#disassembleReferences
	 * @see org.eso.ohs.persistence.ObjectManager#reassembleReferences
	 */
	public void write(Statement stmt, long id, UnpackedStorableObject obj)
		throws ObjectIOException {

		Phase1Target target = (Phase1Target) obj.getObject();

		String sqlText = buildInsertStmt(target);

		try {
			stmt.executeUpdate(sqlText);
		} catch (SQLException e) {
			stdlog_.error("INSERT error", e);
			throw new ObjectIOException("Error executing write: " + sqlText);
		}
	}

	public static String buildInsertStmt(Phase1Target target) {
		StringBuffer sb = new StringBuffer();
		sb.append(target.getTargetId());
		sb.append(",");
		sb.append(target.getProgId());
		sb.append(",\'");
		sb.append(TextUtils.quoteSqlString(target.getItemName()));
		sb.append("\',");
		sb.append(target.getRa());
		sb.append(",");
		sb.append(target.getDeclination());
		sb.append(",");
		sb.append(target.getTimeOnTarget());
		sb.append(",\'");
		sb.append(TextUtils.quoteSqlString(target.getMagnitude()));
		sb.append("\',\'");
		sb.append(TextUtils.quoteSqlString(target.getDiameter()));
		sb.append("\',\'");
		sb.append(TextUtils.quoteSqlString(target.getUserComment()));
		sb.append("\',");
		sb.append(target.getEquinox());
		sb.append(",\'");
		sb.append(TextUtils.quoteSqlString(target.getRunReferences()));
		sb.append("\'");

		StringBuffer starStmt = insertStarsStmt(target);
		String sqlText =
			"INSERT INTO "
				+ Phase1SelectStmt.targetsTable.get()
				+ " ("
				+ target_fields
				+ " ) \n"
				+ " VALUES ("
				+ sb.toString()
				+ " )\n "
				+ starStmt.toString();
		stdlog_.debug(sqlText);
		return sqlText;
	}

	/**
	 * @param user EsoUser
	 * @param userId long
	 * @return StringBuffer
	 */
	private static StringBuffer insertStarsStmt(Phase1Target target) {
		StringBuffer starStmt = new StringBuffer();
		target.getReferenceStar();
		starStmt
			.append("\n ")
			.append(" INSERT INTO ")
			.append(Phase1SelectStmt.refStarTable.get())
			.append(" (")
			.append(star_fields)
			.append(" ) ")
			.append(" VALUES (")
			.append(target.getTargetId())
			.append(",")
			.append(target.getProgId())
			.append(",\'")
			.append(TextUtils.quoteSqlString(target.getReferenceStar()))
			.append("\')");
		return starStmt;
	}

	/**
	 * Throws an ObjectIOException -- the phase 1 database cannot be
	 * changed from within P2PP.
	 *
	 * @param stmt  JDBC Statement for contacting the dbase
	 * @param id    per-table id of the object to be written.
	 * @param obj   an array: the first element is the BO to be stored.
	 *
	 * @exception ObjectIOException     operation is not permitted exception
	 *
	 * @see org.eso.ohs.persistence.ObjectManager#disassembleReferences
	 * @see org.eso.ohs.persistence.ObjectManager#reassembleReferences
	 */
	public void update(Statement stmt, long id, UnpackedStorableObject obj)
		throws ObjectIOException {

	Phase1Target user = (Phase1Target) obj.getObject();
		update(stmt, user);
	}

	/**
		 * Throws an ObjectIOException -- the phase 1 database cannot be
		 * changed from within P2PP.
		 *
		 * @param stmt  JDBC Statement for contacting the dbase
		 * @param obj   an array: the first element is the BO to be stored.
		 *
		 * @exception ObjectIOException     operation is not permitted exception
		 *
		 * @see org.eso.ohs.persistence.ObjectManager#disassembleReferences
		 * @see org.eso.ohs.persistence.ObjectManager#reassembleReferences
		 */
	public void update(Statement stmt, StorableObject obj)
		throws ObjectIOException {

		Phase1Target target = (Phase1Target) obj;
		String sqlText = buildUpdateStmt(target);

		try {
			stmt.executeUpdate(sqlText);
		} catch (SQLException e) {
			stdlog_.error("INSERT error", e);
			throw new ObjectIOException("Error executing write: " + sqlText);
		}
	}

	public static String buildUpdateStmt(Phase1Target target) {
		StringBuffer sb = new StringBuffer();
		sb.append(ITEM_NAME);
		sb.append(" = \'");
		sb.append(TextUtils.quoteSqlString(target.getItemName()));
		sb.append("\',");
		sb.append(R_A);
		sb.append(" = ");
		sb.append(target.getRa());
		sb.append(",");
		sb.append(DEC);
		sb.append(" = ");
		sb.append(target.getDeclination());
		sb.append(",");
		sb.append(TIME_ON_TARGET);
		sb.append(" = ");
		sb.append(target.getTimeOnTarget());
		sb.append(",");
		sb.append(MAGNITUDE);
		sb.append(" = \'");
		sb.append(TextUtils.quoteSqlString(target.getMagnitude()));
		sb.append(",");
		sb.append(DIAMETER);
		sb.append(" = \'");
		sb.append(TextUtils.quoteSqlString(target.getDiameter()));
		sb.append(",");
		sb.append(EQUINOX);
		sb.append(" = ");
		sb.append(target.getEquinox());
		sb.append(",");
		sb.append(COMMENT);
		sb.append(" = \'");
		sb.append(TextUtils.quoteSqlString(target.getUserComment()));
		sb.append("\',");
		sb.append(RUN_REFERENCES);
		sb.append(" = \'");
		sb.append(TextUtils.quoteSqlString(target.getRunReferences()));
		sb.append("\' ");

		String starUpdateStmt = updateRefStarStmt(target);

		String sqlText =
			starUpdateStmt
				+ " \n "
				+ "UPDATE "
				+ Phase1SelectStmt.targetsTable.get()
				+ " SET "
				+ sb.toString()
				+ " WHERE "
				+ ID
				+ " = "
				+ target.getTargetId()
				+ " AND "
				+ PROG_ID
				+ " = "
				+ target.getProgId();
		stdlog_.debug(sqlText);
		return sqlText;
	}

	/**
	 * Method updateAffilStmt
	 * @param user EsoUser
	 * @param userId long
	 * @return StringBuffer
	 */
	private static String updateRefStarStmt(Phase1Target target) {

		String sqlText =
			"UPDATE "
				+ Phase1SelectStmt.targetsTable.get()
				+ " SET "
				+ REF_STARS_NAME
				+ " = \'"
				+ target.getReferenceStar()
				+ "\' WHERE "
				+ REF_STARS_TARGET_ID
				+ " = "
				+ target.getTargetId()
				+ " AND "
				+ REF_STARS_TARGET_ID
				+ " = "
				+ target.getProgId();
		return sqlText;

	}

	/**
	 * Throws an ObjectIOException -- the phase 1 database cannot be
	 * changed from within P2PP.
	 *
	 * @param stmt  JDBC Statement for contacting the dbase
	 * @param id    per-table id of the object to be written.
	 *
	 * @exception ObjectIOException     operation is not permitted exception
	 */
	public void delete(Statement stmt, long id) throws ObjectIOException {
		throw new ObjectIOException("Operation not supported for this object");
	}

	/**
	 * Method accept
	 * @param v BusinessVisitor
	 * @param arg Object
	 * @return Object
	 */
	public Object accept(BusinessVisitor v, Object arg) {
		return null;
	}

	/**
	 * Return the one and only instance of the DbaseHandlerEsoUser.
	 *
	 * @return Singleton DbaseHandlerEsoUser instance.
	 */
	public static DbaseHandlerPhase1Target getDbaseHandlerEsoUser() {
		if (singleton_ == null) {
			singleton_ = new DbaseHandlerPhase1Target();
		}
		return singleton_;
	}
}
