package org.eso.ohs.persistence.dbase.phase2;

import java.io.IOException;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;

import org.eso.ohs.core.utilities.ObjectIOException;
import org.eso.ohs.core.utilities.SqlUtils;
import org.eso.ohs.dfs.Mask;
import org.eso.ohs.dfs.ObservationBlock;
import org.eso.ohs.persistence.Config;
import org.eso.ohs.persistence.ObjectNotFoundException;
import org.eso.ohs.persistence.StorageManager.UnpackedStorableObject;
import org.eso.ohs.persistence.dbase.DbaseIOHandler;
/**
 * I/O class for Masks stored in the database.
 *
 * <p>
 * $Id: DbaseHandlerMask.java,v 1.1 2005/03/30 09:56:12 tcanavan Exp $
 * <br>
 * @author Bob Kemp
 * 
 * @see org.eso.ohs.persistence.dbase.phase2.DbaseMgr
 */

public class DbaseHandlerMask extends DbaseIOHandler {

    private static String obrepDbName_ = Config.getCfg().getDbName(
					       ObservationBlock.getSuffix());            

    /**
     * Constructor for CS I/O class.
     */
    public DbaseHandlerMask() {
    }


    /**
     * Managed class
     * @return Mask class
     */

    public Class getManagedClass() {
	return Mask.class;
    }

    /**
     * Reads the object from the database, partially reassembling it.
     * References to other business objects are stored in other fields
     * of the array
     * 
     * @param stmt  JDBC Statement for contacting the dbase
     * @param id    per-table id of the object to be retrieved.
     * @return	    array in the format required by reassembleReferences()
     * 
     * @exception SQLException
     *		    an error occurred during the dbase operation
     * @exception ObjectNotFoundException
     *		    the requested object was not found in the database
     * @exception IOException
     *              if an I/O error occurred while reading the object
     * 
     * @see org.eso.ohs.persistence.ObjectManager#disassembleReferences
     * @see org.eso.ohs.persistence.ObjectManager#reassembleReferences
     */
    public UnpackedStorableObject read(Statement stmt, long id)
			throws ObjectNotFoundException,
				IOException, SQLException {
	ResultSet rs = null;
	Mask msk = (Mask)newInstance();
	UnpackedStorableObject uso = new UnpackedStorableObject(msk);

        stmt.execute("set TEXTSIZE " + 1000000);
	String sql =    "SELECT * FROM " +obrepDbName_ + "..masks"
                      + " WHERE mask_id = " + id;
			

	System.out.println("CLASS DbaseHandlerMask - read: id = " + id);
	System.out.println("CLASS DbaseHandlerMask - read: sql = " + sql);

	rs = stmt.executeQuery(sql);

	if ( ! rs.next()) { // Only one with this id !!
	    throw new ObjectNotFoundException("Object not found in database");
	}

	msk.setId(Config.getCfg().tableToUniqueId(id, Mask.class));
	//msk.setId(id);
	msk.setMaskNAID(rs.getString("mask_naid"));
	msk.setMaskId(rs.getString("bar_code"));
	msk.setSlotNum(rs.getInt("mask_slot_num"));        
        msk.setADMFile(rs.getString("adm_file"));
        msk.setStatus(rs.getString("mask_status"));
        msk.setName(rs.getString("mask_name"));
	rs.close();

	return uso;
    }


    /**
     * Writes the object to the database, for the first time.
     * 
     * @param stmt  JDBC Statement for contacting the dbase
     * @param id    per-table id of the object to be written.
     * @param obj   an array: the first element is the BO to be stored.
     * @exception SQLException  an error occurred during the dbase operation
     * 
     * @see org.eso.ohs.persistence.ObjectManager#disassembleReferences
     * @see org.eso.ohs.persistence.ObjectManager#reassembleReferences
     */
    public void write(Statement stmt, long id, UnpackedStorableObject obj)
			throws SQLException {
	Mask msk = (Mask)obj.getObject();
System.out.println("CLASS DbaseHandlerMask - write:");
	StringBuffer sb = new StringBuffer();

        sb.append(id);
        sb.append(",");
	sb.append(SqlUtils.sqlEscapedString(msk.getMaskNAID()));
	sb.append(",");
	sb.append(SqlUtils.sqlEscapedString(msk.getMaskId()));
	sb.append(",");
	sb.append(msk.getSlotNum());
        sb.append(",");
        sb.append(SqlUtils.sqlEscapedString(msk.getADMFile()));
        sb.append(",");
        sb.append(SqlUtils.sqlEscapedString(msk.getStatus()));
        sb.append(",");
        sb.append(SqlUtils.sqlEscapedString(msk.getName()));


	String sqlText = "INSERT INTO " + obrepDbName_ + "..masks"			    
			    +   "(mask_id,mask_naid,bar_code, mask_slot_num,adm_file,mask_status,mask_name)"
			    + " VALUES (" + sb + ")";

	System.out.println("CLASS DbaseHandlerMask - write: sqlText = "+sqlText);
	stmt.executeUpdate(sqlText);
    }


    /**
     * Updates an object that is already stored in the database.
     * 
     * @param stmt  JDBC Statement for contacting the dbase
     * @param id    per-table id of the object to be written.
     * @param obj   an array: the first element is the BO to be stored.
     * @exception SQLException  an error occurred during the dbase operation
     * 
     * @see org.eso.ohs.persistence.ObjectManager#disassembleReferences
     * @see org.eso.ohs.persistence.ObjectManager#reassembleReferences
     */
    public void update(Statement stmt, long id, UnpackedStorableObject obj)
			throws SQLException {
	Mask msk = (Mask)obj.getObject();

	StringBuffer sb = new StringBuffer();

        sb.append(" mask_naid = ");
	sb.append(SqlUtils.sqlEscapedString(msk.getMaskNAID()));      
	sb.append(", bar_code = ");
	sb.append(SqlUtils.sqlEscapedString(msk.getMaskId()));       
	sb.append(", mask_slot_num = ");
	sb.append(msk.getSlotNum());      
	sb.append(", adm_file = ");
        sb.append(SqlUtils.sqlEscapedString(msk.getADMFile()));    
	sb.append(", mask_status = ");
	sb.append(SqlUtils.sqlEscapedString(msk.getStatus()));
	sb.append(", mask_name = ");
	sb.append(SqlUtils.sqlEscapedString(msk.getName()));
        


        String sqlText = "UPDATE " + obrepDbName_ + "..masks"
			+ " SET " + sb

                        + " WHERE mask_id = " + id;
	stmt.executeUpdate(sqlText);
    }


    /**
     * Deletes an object from the database.
     * 
     * @param stmt  JDBC Statement for contacting the dbase
     * @param id    per-table id of the object to be written.
     * @exception SQLException  an error occurred during the dbase operation
     */
    public void delete(Statement stmt, long id)

			throws ObjectIOException {
	    throw new ObjectIOException("Operation not supported for this object");

    }

}


