package org.eso.ohs.phase2.actions;
import java.awt.Dimension;
import java.awt.event.ActionEvent;

import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JOptionPane;

import org.eso.ohs.core.gui.baseaction.ActionSuperclass;
import org.eso.ohs.core.gui.baseaction.IncrementalAction;
import org.eso.ohs.core.gui.baseaction.IncrementalActionAutomaton;
import org.eso.ohs.core.gui.widgets.ErrorMessages;
import org.eso.ohs.core.gui.widgets.TextDisplayWidget;
import org.eso.ohs.core.utilities.ObjUtils;
import org.eso.ohs.core.utilities.ObjectIOException;
import org.eso.ohs.core.utilities.TextUtils;
import org.eso.ohs.dfs.BusinessObject;
import org.eso.ohs.dfs.CalibrationBlock;
import org.eso.ohs.persistence.Config;
import org.eso.ohs.persistence.Media;
import org.eso.ohs.persistence.ObjectContainer;
import org.eso.ohs.persistence.ObjectManager;
import org.eso.ohs.persistence.ObjectNotFoundException;

/**
 * Action class to display the detailed contents of a BusinessObject.
 *
 * <p>
 * $Id: OBReportAction.java,v 1.12 2005/11/15 20:14:36 tcanavan Exp $
 * <br>
 * @author Bob Kemp
 */

public class OBReportAction extends ActionSuperclass {


    private static final int ID_PADDING = 8;
    private static final int OB_NAME_PADDING = 26;   
    

    private static TextDisplayWidget fileMenu_ = new TextDisplayWidget("OB Report");

    protected ObjectContainer folderView_;

    protected boolean folder_;
   

    /**
     * Constructs a ShowDetailsAction action.
     */
    public OBReportAction(ObjectContainer folderView,boolean folder,String name) {
	super((JComponent)folderView,name);

	folderView_ = folderView;
        folder_ = folder;
        //fileMenu_.setVisible(false);
    }

    /**
     * Event handler for the ShowDetailsAction action.
     * 
     * @param ev       user-initiated ActionEvent to quit.
     */
    public void actionPerformedImpl(ActionEvent evt) {
        Long [] idLongs = null;        
	if(!folder_)
	    idLongs = folderView_.getAllSelected();
	 else 
	    idLongs = folderView_.getAll();      
	idLongs = (Long[])ObjUtils.uniqueOccurrences(idLongs);
        if(idLongs.length <= 0) {
           JOptionPane.showMessageDialog(
		folderView_.getTopLevelAncestor(),
		new Object[] {
		  "There are no selected business objects",
		  " ",
		  "You must first select business objects to report on."
		},
		"Nothing selected",
		JOptionPane.ERROR_MESSAGE
	    );
	    return;
        }

        new IncrementalActionAutomaton(
			new IncrementalReport(		    			 
			    idLongs
			),
			(JFrame)folderView_.getTopLevelAncestor(),
			"Reports..."
		    );        
    }


    /**
     * Utility class to export a sequence of Business Objects, one at a time.
     * The user is also given the opportunity to stop the process using
     * a dialog box.
     */
    public class IncrementalReport implements IncrementalAction {

	/** cached reference to Persistence */
	private ObjectManager objMgr = ObjectManager.getObjectManager();

	/** array of ids of objects to be duplicated */
	private Long[] idLongs_;

	/** Type of business object to be duplicated */
	//private Class objType_;

	/** Media containing the objects to be duplicated */
	private Media device_;
    

        private int numProcessed;
           
 
        StringBuffer buf = new StringBuffer();

	/**
	 * Constructor.
	 * 
	 * @param device    media containing the objects to be duplicated
	 * @param objType   type of object to be duplicated
	 * @param idLongs   array of ids of objects to be duplicated
	 */
	public IncrementalReport(Long[] idLongs) {	
	    idLongs_  = idLongs;
            
	}

	public int getNumStepsRemaining(int counter) {
	    return ObjUtils.howManyNotNull(idLongs_, counter);
	}

	public void startOfAction() {
            fileMenu_.clearTextArea();          
            fileMenu_.setVisible(false);                     
            numProcessed = 0;         
            buf.append("OBSERVATION BLOCK BREAKDOWN " + "(" + TextUtils.currentISODateTime() + "(UT))\n");
            buf.append("=====================================================\n");
            buf.append("\n");
            buf.append("\n");         
        }

	public void endOfAction() { 
            fileMenu_.setTextArea(buf.toString());
            fileMenu_.setSize(new Dimension(600,400));  
            fileMenu_.setVisible(false);               
            fileMenu_.setVisible(true);           
        }    

	/**
	 * Method to duplicate the n'th element of obs, where n = counter.
	 * 
	 * @param counter   counter to show which element of obs to check in.
	 * @return	    true if the duplicating should continue
	 *		    and false otherwise.
	 */
	public void performStep(int counter) {
	    if (idLongs_[counter] != null) {
				// Main body of the "loop" + fall thru
		doOBReport(idLongs_[counter].longValue());
	    }
	}

	/**
	 * Utility method to generate a report on a business object
	 * 
	 * @param id	    id of business object to duplicate
	 */
	public void doOBReport(long id) {
	     try {
	             Media device = Config.getCfg().getMediaFromId(id);
                     Class objType = Config.getCfg().getClassFromId(id);
                     // refresh the object from the database 
                     // if it is alreadz cached. 
                     if(objMgr.isInRegistry(id)) { 
                    	 objMgr.refreshBusObjRecursive(device,id,objType);
                     }  	 
                     BusinessObject obj  = (BusinessObject)objMgr.getBusObj(device,id,objType);
                     long dbId=-1;
                     if (obj.getId() < 0) { // local
                         if(obj.getDbaseId() != 0) {
                             dbId = Config.getCfg().uniqueToTableId(obj.getDbaseId());
                         }
                     }    
                     else {
                         dbId = obj.getDbaseId();
                     }
                     if(obj instanceof CalibrationBlock) {    
                           buf.append(((CalibrationBlock)obj).toReport(dbId));                    
                     }                                   
                     numProcessed++;
	     }            	
             catch(ObjectNotFoundException e) {
                 ErrorMessages.announceNoObject(
                         (JFrame)folderView_.getTopLevelAncestor(),
                         e
                 );
             }
             catch(ObjectIOException e) {
                 ErrorMessages.announceIOError(
                         (JFrame)folderView_.getTopLevelAncestor(),
                         e
                 );
             }

        }
    }        

}


