package org.eso.ohs.phase2.actions;
import java.awt.event.ActionEvent;

import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JOptionPane;

import org.eso.ohs.core.gui.baseaction.ActionSuperclass;
import org.eso.ohs.core.gui.baseaction.IncrementalAction;
import org.eso.ohs.core.gui.baseaction.IncrementalActionAutomaton;
import org.eso.ohs.core.gui.widgets.ErrorMessages;
import org.eso.ohs.core.gui.widgets.TextDisplayWidget;
import org.eso.ohs.core.utilities.ObjUtils;
import org.eso.ohs.core.utilities.ObjectIOException;
import org.eso.ohs.dfs.BusinessObject;
import org.eso.ohs.persistence.Config;
import org.eso.ohs.persistence.Media;
import org.eso.ohs.persistence.ObjectContainer;
import org.eso.ohs.persistence.ObjectManager;
import org.eso.ohs.persistence.ObjectNotFoundException;


/**
 * Action class to reload the detailed contents of a BusinessObject.
 *
 * @author cmavros 30/11/01
 * 
 */

public class ReloadOBAction extends ActionSuperclass {

    private static TextDisplayWidget fileMenu_ = new TextDisplayWidget("Reload OB");
    protected ObjectContainer folderView_;
    protected boolean folder_;

    /**
     * Constructs a ShowDetailsAction action.
     */
    public ReloadOBAction(ObjectContainer folderView,boolean folder,String name) {
	super((JComponent)folderView,name);
	folderView_ = folderView;
        folder_ = folder;
    }

    /**
     * Event handler for the ShowDetailsAction action.
     * 
     * @param ev       user-initiated ActionEvent to quit.
     */
    public void actionPerformedImpl(ActionEvent evt) {
        Long [] idLongs = null;
	if ( !folder_ ) {
	    idLongs = folderView_.getAllSelected();
	}
	else {
	    idLongs = folderView_.getAll();
	}

	idLongs = (Long[])ObjUtils.uniqueOccurrences(idLongs);      
        //Add error message if nothing selected 09/11/01 CMM
        if(idLongs == null || idLongs.length <= 0) {
           JOptionPane.showMessageDialog(
                folderView_.getTopLevelAncestor(),
                new Object[] {
                  "There are no selected business objects",
                  " ",
                  "You must first select business objects to report on."
                },
                "Nothing selected",
                JOptionPane.ERROR_MESSAGE
            );
            return;
        }
        new IncrementalActionAutomaton(
			new IncrementalReport(
			    idLongs
			),
			(JFrame)folderView_.getTopLevelAncestor(),
			"Reload..."
       );        
    }


    /**
     * Utility class to export a sequence of Business Objects, one at a time.
     * The user is also given the opportunity to stop the process using
     * a dialog box.
     */
    public class IncrementalReport implements IncrementalAction {

	/** array of ids of objects to be duplicated */
	private Long[] idLongs_;

	/** Type of business object to be duplicated */
	private Class objType_;

	/** Media containing the objects to be duplicated */
	private Media device_;


	/**
	 * Constructor.
	 * 
	 * @param device    media containing the objects to be duplicated
	 * @param objType   type of object to be duplicated
	 * @param idLongs   array of ids of objects to be duplicated
	 */
	public IncrementalReport(Long[] idLongs) {
	    idLongs_  = idLongs;            
	}

	public int getNumStepsRemaining(int counter) {
	    return ObjUtils.howManyNotNull(idLongs_,counter);
	}

	public void startOfAction() { }

	public void endOfAction() { 
           JOptionPane.showMessageDialog(
                folderView_.getTopLevelAncestor(),
                new Object[] {
                  "Reload complete.", "", ""
                },
                "Reload",
                JOptionPane.INFORMATION_MESSAGE
            );
        }


	/**
	 * Method to duplicate the n'th element of obs, where n = counter.
	 * 
	 * @param counter   counter to show which element of obs to check in.
	 * @return	    true if the duplicating should continue
	 *		    and false otherwise.
	 */
	public void performStep(int counter) {
	    if (idLongs_[counter] != null) {
	        // Main body of the "loop" + fall thru
		doReloadOBs(idLongs_[counter].longValue());
	    }
	}

        /**
         *
	 * Utility method to refresh an OB that might have 
         * changed.
	 *
	 * @param id id of business object to duplicate
         * 
         */
        public void doReloadOBs(long id) {
            try {
                device_ = Config.getCfg().getMediaFromId(id);
                objType_ = Config.getCfg().getClassFromId(id);
                ObjectManager objMgr = ObjectManager.getObjectManager();
                objMgr.refreshBusObjRecursive(device_,id,objType_);
                BusinessObject obj  = (BusinessObject)objMgr.getBusObj
                  (device_,id,objType_);
            }
            catch(ObjectNotFoundException e) {
                ErrorMessages.announceNoObject(
                  (JFrame)folderView_.getTopLevelAncestor(), e);
            }
            catch(ObjectIOException e) {
                ErrorMessages.announceIOError(
                  (JFrame)folderView_.getTopLevelAncestor(), e);
            }
            catch (RuntimeException e) {
                e.printStackTrace();
                throw e;
            }
            catch (Exception e) {
                e.printStackTrace();
            }
        }

    }        

}
