package org.eso.ohs.phase2.apps.p2pp.actions;

import java.awt.event.ActionEvent;
import java.lang.reflect.InvocationTargetException;

import javax.swing.JComponent;
import javax.swing.JOptionPane;

import org.eso.ohs.core.gui.baseaction.ActionSuperclass;
import org.eso.ohs.core.utilities.ObjectIOException;
import org.eso.ohs.dfs.BusinessObject;
import org.eso.ohs.dfs.ConstraintProperties;
import org.eso.ohs.dfs.ObservationBlock;
import org.eso.ohs.dfs.ObservingRun;
import org.eso.ohs.instruments.InstrumentList;
import org.eso.ohs.p2pp.Folder;
import org.eso.ohs.persistence.AppConfig;
import org.eso.ohs.persistence.Media;
import org.eso.ohs.phase2.apps.p2pp.FolderGridView;
import org.eso.ohs.phase2.apps.p2pp.Persistence;
/**
 * Action class to create a business object in the chosen directory.
 * 
 * <p>
 *
 * $Id: CreateBOAction.java,v 1.12 2005/11/02 11:23:08 tcanavan Exp $
 *
 */

public class CreateBOAction extends ActionSuperclass {
    /** medium on which the business object is to be created */
    Media device_;

    /** view of the folder in which the OB is to be created */
    FolderGridView folderView_;

    /**
     * Constructs a create BusinessObject action.
     * 
     * @param device      medium containing the parent directory.
     * @param folderView  GUI view of (summary information in) the folder.
     */
    public CreateBOAction(Media device, FolderGridView folderView) {
        super((JComponent) folderView, "New");
        device_ = device;
        folderView_ = folderView;
    }

    /**
     * Event handler for the create BusinessObject action.
     * 
     * @param ev       user-initiated ActionEvent to create an OD.
     */
    public void actionPerformedImpl(ActionEvent ev) {
        try {
            Folder folder = folderView_.getFolder();
            Class objType = folderView_.getDisplayedObjectClass();
            BusinessObject bo = null;

            if (folder == null) {
                JOptionPane.showMessageDialog(
                    folderView_.getTopLevelAncestor(),
                    new Object[] {
                        "There is no selected folder.",
                        "Where should the new object be created?",
                        },
                    "Nothing selected",
                    JOptionPane.ERROR_MESSAGE);
                return;
            }

            ObservingRun obr = folder.findObsRun();
            String instCode = obr.getInstCode();

            if (AppConfig.getAppConfig().isIPVersioningEnabled()) {
                if (!InstrumentList
                    .getInstance()
                    .instrumentExists(obr.getInstCode(), obr.getIPVersion())) {
                    JOptionPane.showMessageDialog(
                        (JComponent) folderView_,
                        "Unable to create a new OB as the Instrument Package '"
                            + obr.getInstCode()
                            + " version "
                            + obr.getIPVersion()
                            + "' is not installed",
                        "Cannot create Observation Description",
                        JOptionPane.WARNING_MESSAGE);
                    return;
                }
            } else {
                if (!InstrumentList
                    .getInstance()
                    .instrumentExists(obr.getInstCode())) {
                    JOptionPane.showMessageDialog(
                        (JComponent) folderView_,
                        "Instrument Package "
                            + obr.getInstCode()
                            + " not installed",
                        "Cannot create Observation Description",
                        JOptionPane.WARNING_MESSAGE);
                    return;
                }
            }

            Persistence objMgr = Persistence.getInstance();

            bo =
                objMgr.createAndRegisterRecursive(
                    device_,
                    folder.getNode(),
                    objType);
        } catch (ObjectIOException e) {
            announceIOError(e);
        } 
    }

}
