package org.eso.ohs.phase2.apps.p2pp.actions;

import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.Hashtable;

import javax.swing.JComponent;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JOptionPane;

import org.apache.log4j.Logger;
import org.eso.ohs.core.gui.baseaction.ActionSuperclass;
import org.eso.ohs.core.gui.widgets.ErrorMessages;
import org.eso.ohs.core.gui.widgets.ExtensionFileFilter;
import org.eso.ohs.core.gui.widgets.SwingFileChooser;
import org.eso.ohs.core.gui.widgets.TextDisplayWidget;
import org.eso.ohs.core.utilities.ObjectIOException;
import org.eso.ohs.core.utilities.PafFile;
import org.eso.ohs.dfs.ObservingRun;
import org.eso.ohs.dfs.Readme;
import org.eso.ohs.instruments.Instrument;
import org.eso.ohs.instruments.InstrumentList;
import org.eso.ohs.p2pp.Folder;
import org.eso.ohs.persistence.AppConfig;
import org.eso.ohs.persistence.ImportReadme;
import org.eso.ohs.phase2.apps.p2pp.HierarchyView;
import org.eso.ohs.phase2.apps.p2pp.MainWindow;

/**
 * Action class to create a business object in the chosen directory.
 * 
 * <p>
 * $Id: ImportReadmeAction.java,v 1.13 2005/02/11 11:00:27 tcanavan Exp $
 * 
 * @author cmavros 05JUNE02 [1766/P2PP] Do not import files with blank name
 *         fields.
 */

public class ImportReadmeAction extends ActionSuperclass {

    /** Standard log channel */
    private static Logger stdlog_ = Logger.getLogger(ImportBOAction.class);

    private HierarchyView hView_;

    private ObservingRun obsRun_;

    /**
     * Return value from dialog box
     */

    private String returnValue_;

    /**
     * A hashtable of strings that contain any waring information about the
     * import of an individual file
     *  
     */

    private Hashtable warningList_ = new Hashtable();

    /**
     * Status window which displays reports on imports.
     */

    private static TextDisplayWidget log_ = new TextDisplayWidget(
            "Import Report");

    /**
     * Static reference to file chooser dialog
     */

    private static SwingFileChooser chooser = new SwingFileChooser(AppConfig
            .getAppConfig().getExpDir());

    /**
     * Static refrences to file chooser filters
     */

    private static ExtensionFileFilter rmFilter = new ExtensionFileFilter(
            "rdx", "Readme Import Files");

    static {
        File f1 = new File(AppConfig.getAppConfig().getExpDir());
        chooser.setCurrentDirectory(f1);
        chooser.setSelectedFile(f1);
    }

    /**
     * Constructs a create BusinessObject action.
     * 
     * @param device
     *            medium containing the parent directory.
     * @param folderView
     *            GUI view of (summary information in) the folder.
     */

    public ImportReadmeAction(HierarchyView hView) {
        super((JComponent) hView, "Import Readme");
        hView_ = hView;
    }

    /**
     * Event handler for the create BusinessObject action.
     * 
     * @param ev
     *            user-initiated ActionEvent to create an OD.
     */

public void actionPerformedImpl(ActionEvent ev) {
        chooser.addChoosableFileFilter(rmFilter);

        chooser.setFileFilter(rmFilter);

        // Java1.4 port chooser.clearSelection();
        chooser.cancelSelection();
        chooser.setMultiSelectionEnabled(true);
        int response = chooser.showDialog(null, "Select Import File");
        if (response == JOptionPane.CLOSED_OPTION
            || response == JOptionPane.CANCEL_OPTION)
            return;

        File f = chooser.getSelectedFile();
        if(f !=null)
        stdlog_.debug("Absolute path  " + f.getAbsolutePath());
        if (response == JFileChooser.APPROVE_OPTION) {
            Object momento = MainWindow.getView().hideViews();
            try {
                Folder folder;
                folder = hView_.getSelectedFolder();
                ObservingRun obsRun = folder.findObsRun();
                if (checkReadme(f, obsRun)) {
                    doImport(f, obsRun);
                }
            }      
            catch (FileNotFoundException e) {
                ErrorMessages.announceIOError(hView_, e);
            }   catch (IOException e) {
                ErrorMessages.announceIOError(hView_, e);
            }   catch (ObjectIOException e1) {
                ErrorMessages.announceIOError(hView_, e1);
            }
        }   
    }    
        
        
        public void doImport(File file, ObservingRun obsRun)
            throws FileNotFoundException, IOException {
        ImportReadme readme = new ImportReadme();
        readme.importReadme(obsRun, new PafFile(file.getAbsolutePath(), false,
                true));
        log_.setSize(new Dimension(700, 300));
        log_.clearTextArea();
        log_.setVisible(true);
        if(readme.hasWarning()) {
            String buf = "Readme scuccessfully imported with the following warnings \n" + readme.warningReport(); 
            log_.setTextArea(buf);
        } else {
            log_.setTextArea("Readme Imported Without Warning/Errors");
        }       

        
    }

    private boolean checkReadme(File file, ObservingRun obsRun)
            throws IOException, FileNotFoundException {
        if (!file.canRead()) {
            JOptionPane.showMessageDialog(hView_.getTopLevelAncestor(),
                    "You do not have read permissions on file "
                            + file.getName(), "Read Error",
                    JOptionPane.WARNING_MESSAGE);
            return false;
        }
        String filename = file.getAbsolutePath();
        PafFile pafFile = new PafFile(filename, false, true);
        if (pafFile.compileError()) {
            JOptionPane.showMessageDialog(hView_.getTopLevelAncestor(),
                    "This file has an invlid format", "Invalid file format",
                    JOptionPane.WARNING_MESSAGE);
            return false;
        }
        if (!InstrumentList.getInstance()
                .instrumentExists(obsRun.getInstCode())) {
            JOptionPane.showMessageDialog(hView_.getTopLevelAncestor(),
                    "Cannot import into an observing run which has "
                            + "no associated instrument package installed ",
                    "Instrument package not installed",
                    JOptionPane.WARNING_MESSAGE);
            return false;
        }
        Instrument inst = InstrumentList.getInstance().getInstrument(
                obsRun.getInstCode(), obsRun.getIPVersion());
        if (!inst.hasReadmeData()) {
            JOptionPane
                    .showMessageDialog(
                            hView_,
                            new Object[] {
                                    "There is no readme configuration available for this observing run.",
                                    "Try download refresh observing runs",
                                    "If this still does not work then contact usd-help@eso.org", },
                            "Readme Available", JOptionPane.ERROR_MESSAGE);
            return false;
        }
        Readme readme = obsRun.getReadme();
        if (readme == null) {
            readme = Readme.createReadme(inst);
            // fire event.
            obsRun.setReadme(readme);
        } else {
            if (readme.isCheckedIn()) {
                JOptionPane.showMessageDialog(hView_.getTopLevelAncestor(),
                        "Readme file not imported. It is already checked-in",
                        "Already Checked-in", JOptionPane.WARNING_MESSAGE);
                return false;
            }

            int returnValue = JOptionPane.showConfirmDialog((JFrame) hView_
                    .getTopLevelAncestor(),
                    "Do you wish to overwrite the current readme file ?");
            if (returnValue != JOptionPane.YES_OPTION)
                return false;

        }
        return true;
    }
}