// $Id: OBView.java,v 1.49 2005/04/14 09:41:30 tcanavan Exp $

package org.eso.ohs.phase2.apps.p2pp.views;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.IOException;
import java.lang.reflect.InvocationTargetException;
import java.sql.SQLException;

import javax.swing.Box;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.ListSelectionModel;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.table.TableModel;

import org.apache.log4j.Logger;
import org.eso.ohs.core.docview.gui.DocView;
import org.eso.ohs.core.gui.models.BoNameModel;
import org.eso.ohs.core.gui.models.LimitedCharsModel;
import org.eso.ohs.core.gui.widgets.ErrorMessages;
import org.eso.ohs.core.gui.widgets.ToolTipJList;
import org.eso.ohs.core.utilities.Convert;
import org.eso.ohs.core.utilities.OHSConfig;
import org.eso.ohs.core.utilities.OHSKeys;
import org.eso.ohs.core.utilities.ObjectIOException;
import org.eso.ohs.dfs.BusinessObject;
import org.eso.ohs.dfs.CalibrationBlock;
import org.eso.ohs.dfs.ConstraintProperties;
import org.eso.ohs.dfs.ObservationBlock;
import org.eso.ohs.dfs.ObservationDescription;
import org.eso.ohs.dfs.ObservingRun;
import org.eso.ohs.dfs.TimeInterval;
import org.eso.ohs.evm.EVMException;
import org.eso.ohs.evm.EvmExecutorImpl;
import org.eso.ohs.instruments.Instrument;
import org.eso.ohs.instruments.InstrumentList;
import org.eso.ohs.instruments.TemplateSignature;
import org.eso.ohs.instruments.TemplateSignatureConstants;
import org.eso.ohs.instruments.TemplateSignatureFactory;
import org.eso.ohs.persistence.AppConfig;
import org.eso.ohs.persistence.Config;
import org.eso.ohs.persistence.OHSApplication;
import org.eso.ohs.persistence.ObjectNotFoundException;
import org.eso.ohs.persistence.dbase.phase2.OBHistory;
import org.eso.ohs.phase2.apps.p2pp.MainWindow;
import org.eso.ohs.phase2.apps.p2pp.Persistence;

/**
 * This is a specialised JPanel with subcomponents linked to the
 * given ObservationBlock object's fields.  When this component is
 * displayed, changes to it will be reflected in the ObservationBlock
 * object.  Likewise changes to the ObservationBlock object will be
 * reflected in the displayed fields.  Some fields of an ObservationBlock
 * may not be visible.  Some fields may not be modifiable.
 *
 * <p>
 * $Id: OBView.java,v 1.49 2005/04/14 09:41:30 tcanavan Exp $
 *
 * <br>ChangeLog: cmavros added 30/Aug/2001
 * Add Textfield for Execution Time
 * <br>ChangeLog: cmavros added 20-11-01
 * move Execution Time calculation to utilities XTime
 */

public class OBView extends ObjectView implements PropertyChangeListener {

    /** Standard log channel */
    private static Logger stdlog_ = Logger.getLogger(OBView.class);

    private static String[] userPriority_ =
        { "1", "2", "3", "4", "5", "6", "7", "8", "9", "10" };

    public static JTextField executionTextField_;

    /** ObservationBlock currently displayed, or null */
    private ObservationBlock currentOB_;

    /* GUI component for the Name field */
    private DocView fieldName_;

    /* GUI component for the Standard field */
    // private DocView  fieldStandard_;

    /* GUI component for the UserPriority field */
    private DocView fieldUserPriority_;

    /* GUI component for the Status field */
    private DocView fieldStatus_;

    /* GUI component for the UserComments field */
    private DocView fieldUserComments_;
    
    private DocView fieldInstComments_;


    /* GUI component for the CalibrationRequirements field */
    private DocView fieldCalibrationRequirements_;

    /* GUI component for the OD Name field */
    private DocView fieldODName_;

    /* GUI component for the Execution Time field */
    /* cmavros added 30/Aug/2001 */
    private DocView fieldExecutionTime_;

    /* Button to allow the acquisition template to be edited */
    private JButton editTemplates_;

    /** Button to allow the user to update the object on external media */
    private JButton save_;
    
    private boolean hasInstComments=false;

    /**
     * Button to recaluate the execution time when a parameter changes.
     */

    private JButton execTimeButton_;

    private JButton duplicateTemplate_;

    /** List for template names */
    private ToolTipJList templateNames_;

    /** Add template button*/
    private JButton addTemplate_;

    private JTabbedPane tabPane_;

    /** Remove template button*/
    private JButton removeTemplate_;

    /** Table that contains Acquisition model for OBView*/
    private OBTemplateTable templateTable_;

    /** Acquisition model for OBView*/
    private AcquisitionModel templateModel_;

    /** Label for name of Observation Block*/
    private JLabel nameLabel_;

    /** Label for name of Observation Block*/
    private JLabel execTimeLabel_;
    
    
    private JLabel instComments_;

    /** Reference to the currently selected instrument*/
    private Instrument inst_;

    /** Nested ObjectView to display the Target for this OB */
    private TargetView targetView_;

    /** Nested ObjectView to display the ConstraintSet for this OB */
    private CSView csView_;

    /** Nested ObjectView to display the TimeIntervals for this OB */
    private TIView tiView_;
    
    /** Nested ObjectView to display the STTimeIntervals for this OB */
    private TIView stView_;


    /** Should the target be displayed? */
    private boolean showTarget_ = true;

    private boolean edit_;

    private JList instrumentTypes_ = new JList();

    /** cmavros 13-11-01 Preferred size for View */
    private Dimension size = new Dimension(250, 300);

    private void setBorder(JPanel panel, Color color) {
        // panel.setBorder(BorderFactory.createLineBorder(color));
    }

    /**
     * Construct the JPanel and add the subcomponents and listeners, etc.
     * Some fields of an ObservationBlock may not be visible.
     * Some fields may not be modifiable.
     *
     * <p> Initially, no OB is displayed.
     */
    public OBView() {

        JPanel outerPanel = new JPanel(new BorderLayout());
        outerPanel.setBorder(new javax.swing.border.EmptyBorder(4, 4, 4, 4));

        setLayout(new BorderLayout());
        setBorder(outerPanel, Color.red);
        add(outerPanel, BorderLayout.NORTH);

        JPanel topPanel = new JPanel(new GridLayout(1, 0));
        outerPanel.add(topPanel, BorderLayout.NORTH);
        JPanel userCommentsPanel = new JPanel(new BorderLayout());
        outerPanel.add(userCommentsPanel,BorderLayout.CENTER);
        JPanel instCommentsPanel = new JPanel(new BorderLayout());
        outerPanel.add(instCommentsPanel,BorderLayout.SOUTH);

        
        JPanel topCol1 = new JPanel(new GridLayout(0, 1));
        JPanel topCol2 = new JPanel(new BorderLayout());
        topPanel.add(topCol1);
        topPanel.add(topCol2);
        setBorder(topCol1, Color.cyan);
        setBorder(topCol2, Color.black);
        JPanel outerNmStatus = new JPanel(new BorderLayout());
        topCol1.add(outerNmStatus);

        JPanel leftCol1 = new JPanel();
        JPanel rightCol1 = new JPanel();
        leftCol1.setLayout(new GridLayout(0, 1));
        rightCol1.setLayout(new GridLayout(0, 1));
        outerNmStatus.add(leftCol1, BorderLayout.WEST);
        outerNmStatus.add(rightCol1, BorderLayout.CENTER);

        JPanel leftCol2 = new JPanel();
        JPanel rightCol2 = new JPanel();
        //leftCol2.setLayout(new GridLayout(0,1));
        //rightCol2.setLayout(new GridLayout(0,1));
        leftCol2.setLayout(new BorderLayout());
        rightCol2.setLayout(new BorderLayout());
        topCol2.add(leftCol2, BorderLayout.WEST);
        topCol2.add(rightCol2, BorderLayout.CENTER);
        setBorder(leftCol1, Color.magenta);
        setBorder(rightCol1, Color.green);
        setBorder(outerNmStatus, Color.yellow);

        /* ObsBlock Name */
        nameLabel_ = new JLabel("Name: ");
        nameLabel_.setHorizontalAlignment(JLabel.RIGHT);
        leftCol1.add(nameLabel_);
        JTextField tf = new JTextField(Config.getCfg().getMaxNameLength());
        tf.setDocument(new BoNameModel());
        fieldName_ = new DocView(tf);
        rightCol1.add(fieldName_) ;

        /* Status */
        JLabel lab = new JLabel("Status: ");
        lab.setHorizontalAlignment(JLabel.RIGHT);
        leftCol1.add(lab);
        tf = new JTextField();
        tf.setEditable(false);
        tf.setOpaque(false);
        fieldStatus_ = new DocView(tf);
        rightCol1.add(fieldStatus_);

        /* Execution Time */
        /* cmavros added 30/Aug/2001 */
        execTimeLabel_ = new JLabel("Execution Time");
        execTimeLabel_.setHorizontalAlignment(JLabel.RIGHT);

        leftCol1.add(execTimeLabel_);
        executionTextField_ = new JTextField();
        executionTextField_.setEditable(false);
        executionTextField_.setOpaque(false);
        fieldExecutionTime_ = new DocView(executionTextField_);
        rightCol1.add(fieldExecutionTime_);

        /* User Priority */
        lab = new JLabel("User Priority: ");
        lab.setHorizontalAlignment(JLabel.RIGHT);
        leftCol1.add(lab);
        fieldUserPriority_ = new DocView(new JComboBox(userPriority_));
        rightCol1.add(fieldUserPriority_);

        /* OD Name */
        lab = new JLabel("OD Name: ");
        lab.setHorizontalAlignment(JLabel.RIGHT);
        leftCol1.add(lab);
        tf = new JTextField(Config.getCfg().getMaxNameLength());
        tf.setDocument(new BoNameModel());
        fieldODName_ = new DocView(tf);
        rightCol1.add(fieldODName_);

        /* Template Type (i.e. acquisition, science, claib, test)*/
        JPanel instTitlePanel = new JPanel(new GridLayout(0, 1));
        lab = new JLabel("Template Type");
        instTitlePanel.add(lab);
        leftCol2.add(instTitlePanel, BorderLayout.NORTH);
        setBorder(leftCol2, Color.blue);
        setBorder(rightCol2, Color.white);
        //topCol2.add(lab);
        //templateNames_ = new ToolTipJList();

        /* Set a fixed prototypical size of list cell. */
        instrumentTypes_.setPrototypeCellValue("mmmmMmmmmMmmmmM");
        final JScrollPane instScrollPane = new JScrollPane(instrumentTypes_);
        leftCol2.add(instScrollPane, BorderLayout.CENTER);

        /* Template */
        JPanel titlePanel = new JPanel(new GridLayout(0, 1));
        lab = new JLabel("Template");
        titlePanel.add(lab);
        rightCol2.add(titlePanel, BorderLayout.NORTH);
        templateNames_ = new ToolTipJList();

        /* Set a fixed prototypical size of list cell. */
        templateNames_.setPrototypeCellValue("mmmmMmmmmMmmmmM");
        templateNames_.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        final JScrollPane nameScrollPane = new JScrollPane(templateNames_);
        rightCol2.add(nameScrollPane, BorderLayout.CENTER);
        /** cmavros 14-11-01 Add preferred size */
        nameScrollPane.setPreferredSize(
            new Dimension(size.width, size.height - 200));

        /* Buttons: Fourth column of grid */
        initButtons();
        Box buttonPanel = Box.createVerticalBox();
        buttonPanel.add(addTemplate_);
        buttonPanel.add(removeTemplate_);
        buttonPanel.add(duplicateTemplate_);
        buttonPanel.add(execTimeButton_);
        buttonPanel.add(save_);

        buttonPanel.add(Box.createVerticalGlue());
        topCol2.add(buttonPanel, BorderLayout.EAST);

        /* User Comments */
        userCommentsPanel.add(new JLabel("User Comments: "), BorderLayout.NORTH);
        JTextArea ucTA = new JTextArea(1, 80);
        ucTA.setLineWrap(true); // doesn't always work after resizing
        ucTA.setWrapStyleWord(true);
        ucTA.setDocument(new LimitedCharsModel());
        fieldUserComments_ = new DocView(ucTA); 
        // HORIZONTAL_SCROLLBAR_AS_NEEDED doesn't seem to work properly: jdk1.4
        JScrollPane ucSP =
            new JScrollPane(
                fieldUserComments_,
                JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,
                JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
        ucSP.setPreferredSize(ucSP.getPreferredSize());
        userCommentsPanel.add(ucSP, BorderLayout.SOUTH);
        
        
        instComments_ = new JLabel("Instrument Comments: ");
        instCommentsPanel.add(instComments_, BorderLayout.NORTH);
        JTextArea icTA = new JTextArea(1, 80);
        icTA.setLineWrap(true); // doesn't always work after resizing
        icTA.setWrapStyleWord(true);
        icTA.setDocument(new LimitedCharsModel());
        fieldInstComments_ = new DocView(icTA); 
        // HORIZONTAL_SCROLLBAR_AS_NEEDED doesn't seem to work properly: jdk1.4
        JScrollPane icSP =
            new JScrollPane(
                fieldInstComments_,
                JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,
                JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
        icSP.setPreferredSize(icSP.getPreferredSize());
        instCommentsPanel.add(icSP, BorderLayout.SOUTH);

        
        
        
        
        
        
        
        
        
        
        

        /* OB Template Table  */
        templateTable_ = new OBTemplateTable();
        templateTable_.setIsInScrollPane(true);
        add(new JScrollPane(templateTable_));
        /** cmavros 14-11-01 Adjust size */
        templateTable_.setPreferredScrollableViewportSize(
            new Dimension(size.width, size.height));

        /* Tabs (located at the bottom of screen) */
        targetView_ = new TargetView();
        csView_ = new CSView();
        tiView_ = new TIView(Convert.ISO_DATE_FORMAT,TimeInterval.STANDARD_TI_TYPE);
        stView_ = new TIView(Convert.ST_TIME_FORMAT,TimeInterval.SIDEREAL_TI_TYPE);

        tabPane_ = new JTabbedPane();
        /////////
        //Dimension JPSDimT = new Dimension(430, 160);
        //tabPane_.setMaximumSize(JPSDimT);
        //tabPane_.setPreferredSize(JPSDimT);
        /////////
        tabPane_.addTab("Target", targetView_);
        tabPane_.addTab("Constraint Set", csView_);
        tabPane_.addTab("Time Intervals", tiView_);
        tabPane_.addTab("Sidereal Time", stView_);


        /* Tab for Calibration Requirements */
        JTextArea texta = new JTextArea();
        texta.setDocument(new LimitedCharsModel());
        fieldCalibrationRequirements_ = new DocView(texta);
        JScrollPane cReqJSP = new JScrollPane(fieldCalibrationRequirements_);
        Dimension JSPDimT = new Dimension(40, 20);
        cReqJSP.setMaximumSize(JSPDimT);
        cReqJSP.setPreferredSize(JSPDimT);
        tabPane_.addTab("Calibration Requirements", cReqJSP);

        add(tabPane_, BorderLayout.SOUTH);

        addListListeners();
        addActionListeners();
        setObject(null);
    }

    /**
     * Initialise add and remove buttons
     */
    private void initButtons() {
        addTemplate_ = new JButton("Add");
        removeTemplate_ = new JButton("Delete");
        duplicateTemplate_ = new JButton("Duplicate");
        execTimeButton_ = new JButton("Recalc ExecTime");
        save_ = new JButton("Save");
    }

    /**
     * Construct the JPanel and add the subcomponents and listeners, etc.
     * Some fields of an ObservationBlock may not be visible.
     * Some fields may not be modifiable.
     *
     * @param ob the ObservationBlock object to link to.
     */
    public OBView(ObservationBlock ob) {
        this();
        setObject(ob);
    }

    /**
     * Returns a reference to the currently displayed OB.
     *
     * @return reference to an ObservationBlock.
     */
    public BusinessObject getObject() {
        return currentOB_;
    }

    /**
     * Sets a new value of the current OB.
     * This method is included to satisfy the ObjectView
     * interface, which in turn allows for generalisation of
     * other parts of the system.
     *
     * @param newValue reference to a BusinessObject.
     */
    public void setObject(BusinessObject newValue) {
        setObject((ObservationBlock) newValue);
    }

    /**
     * Allows the user to explicitly save the displayed object
     * to external media by making the "Save" button visible.
     */
    public void putInEditMode() {
        setEnabled(true);
        save_.setEnabled(true);
        save_.setVisible(true);
        fieldODName_.setEnabled(true);
        if (csView_ != null)
            csView_.setEnabled(true);
        if (targetView_ != null) {
            targetView_.setEnabled(true);
        }
        if (tiView_ != null)
            tiView_.setEnabled(true);
        templateNames_.setEnabled(false);
    }

    /**
     * Enable or disable the fields of this panel and any
     * nested panels.  The "locked" icon is displayed
     * next to the Name field if the fields are locked.
     *
     * @param enable    True if the fields are to be enabled,
     *          false otherwise.
     */
    public void setEnabled(boolean enable) {
        super.setEnabled(enable);
        nameLabel_.setIcon(enable ? null : locked_);
        fieldName_.setEnabled(enable);
        fieldUserComments_.setEnabled(enable);
        fieldInstComments_.setEnabled(enable);

        fieldCalibrationRequirements_.setEnabled(enable);
        fieldUserPriority_.setEnabled(enable);
        templateNames_.setEnabled(enable);
        fieldODName_.setEnabled(enable);
        targetView_.setEnabled(enable);
        tiView_.setEnabled(enable);
        csView_.setEnabled(enable);
        removeTemplate_.setEnabled(enable);
        duplicateTemplate_.setEnabled(enable);
        //execTimeButton_.setEnabled(enable); 
        if (!enable
            || currentOB_ == null
            || currentOB_.verify().length() > 0) {
            execTimeButton_.setEnabled(false);
        } else {
            execTimeButton_.setEnabled(true);
        }
        if(!hasInstComments)
            fieldInstComments_.setEnabled(false);
            

    }

    /**
     * Initialise template list
     */
    private void initLists() {
        ObservingRun obr = currentOB_.getObsRun();
        if (obr == null)
            return; //XXXX
        inst_ =
            InstrumentList.getInstance().getInstrument(
                obr.getInstCode(),
                obr.getIPVersion());
        instrumentTypes_.setListData(
            TemplateSignatureConstants.allowedTemplateTypes);
        instrumentTypes_.setSelectedIndex(0);
        if (inst_ != null) {
            templateNames_.setListData(
                inst_.getTemplateNames(
                    (String) instrumentTypes_.getSelectedValue()));
            templateNames_.setCellRenderer(new TemplateListCellRenderer(inst_));
        }
    }

    /**
     * Add List Selection listeners for the instrumet type and
     * template name
     */
    private void addListListeners() {
        ListSelectionListener lsl = new ListSelectionListener() {
            public void valueChanged(ListSelectionEvent ev) {
                if (ev.getValueIsAdjusting()) {
                    return;
                }
                String type = (String) instrumentTypes_.getSelectedValue();
                int index;
                if (ev.getSource() == instrumentTypes_) {
                    type = (String) instrumentTypes_.getSelectedValue();
                    addTemplate_.setEnabled(false);
                } else if (ev.getSource() == templateNames_) {
                    index = templateNames_.getSelectedIndex();
                    if (index >= 0)
                        addTemplate_.setEnabled(true);
                    else
                        addTemplate_.setEnabled(false);
                    if (type.equals("acquisition")) {
                        addTemplate_.setToolTipText(
                            "Add "
                                + type
                                + " template to first column of grid");
                    } else
                        addTemplate_.setToolTipText(
                            "Add " + type + " template to the grid");
                } else {
                    stdlog_.debug("Unknown target: event = " + ev);
                }
                if (type != null && ev.getSource() != templateNames_) {
                    if (inst_ == null)
                        return; // XXX
                    templateNames_.setListData(inst_.getTemplateNames(type));
                }
            }
        };
        instrumentTypes_.addListSelectionListener(lsl);
        templateNames_.addListSelectionListener(lsl);

        templateNames_.addMouseListener(new MouseAdapter() {
            public void mouseClicked(MouseEvent evt) {
                if (evt.getClickCount() == 2)
                    addTemplate();
            }
        });
    }

    /**
     * Handle a change in the property value for the associated
     * BusinessObject.  Enable or disable changes to the fields,
     * according to whether they are allowed to be modified.
     * Update nested ObjectViews.
     *
     * @param ev    Property Change Event
     */
    public void propertyChange(PropertyChangeEvent ev) {
        // XXX Should never happen but...
        stdlog_.debug(
            "CLASS OBView - propertyChange: Autsch!!! " + ev.getPropertyName());
        if (currentOB_ == null) {
            stdlog_.debug(
                "@@ ??? Property change for 'null' OB??: " + ev.getSource());
            return;
        }
        if (!Config.getCfg().isInOperationsMode()) {
            setEnabled(isAllowedToModify(currentOB_));
            if (isAllowedToModify(currentOB_)) {
                addTemplate_.setEnabled(!templateNames_.isSelectionEmpty());
            } else {
                addTemplate_.setEnabled(false);
            }

        } else {
            putInEditMode();
            //             stdlog_.debug("CLASS OBView - propertyChange: Autsch!!! putInEditMode()");
        }

        if (ev.getPropertyName() != null) {
            if (ev.getPropertyName().equals("Template")) {
                currentOB_.setExecTimeUpToDate(false);
                execTimeLabel_.setText("* Execution Time");
            }
            if (ev.getPropertyName().equals("ExecutionTime")) {
                execTimeLabel_.setText("Execution Time");
            }
            if(ev.getPropertyName().equals("InstrumentComments")) {
                stdlog_.debug("Comments moved");
                String instCode = currentOB_.getOd().getInstrument();
                Instrument inst = InstrumentList.getInstance().getInstrument(instCode,currentOB_.fetchIPVersionFromOd());
                if(inst.hasInstComments()&&(!currentOB_.isCheckedIn())) {
                    fieldInstComments_.setEnabled(true);
                } else {
                    fieldInstComments_.setEnabled(false);
                }

            }
        }
        
        
    }

    /**
     * Add a template to the observation description
     */
    private void addTemplate() {
        //         stdlog_.debug("CLASS OBView.addTemplate");
        try {
            Object templateName = templateNames_.getSelectedValue();
            TemplateSignature ts =
                TemplateSignatureFactory.getTemplate(
                    inst_,
                    (String) templateName);
            int column = 0;
            if (templateModel_.getColumnCount() != 0) {
                templateTable_.stopCellEditing();
                column = templateTable_.getSelectedColumn();
            }
            templateModel_.insertColumn(column, ts);
            if (templateModel_.getTemplateSignatures() != null)
                currentOB_.getOd().setSignatures(
                    templateModel_.getTemplateSignatures());
            templateTable_.setRowColumn(0, templateModel_.getCurrColumn());
            templateTable_.requestFocus();
        } catch (IOException e) {
            ErrorMessages.announceIOError(templateTable_, e);
        }
    }

    /**
     * Add the menuItem, removeTemplate and addTemplate listeners
     */
    private void addActionListeners() {
        addTemplate_.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                addTemplate();

            }
        });

        removeTemplate_.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                templateTable_.stopCellEditing();
                int column = templateTable_.getSelectedColumn();
                TemplateSignature[] array;

                templateModel_.removeColumn(column);
                int columnCount = templateModel_.getColumnCount();
                if (columnCount > 0) {
                    array = templateModel_.getTemplateSignatures();
                } else {
                    array = new TemplateSignature[0];
                }
                if (array == null)
                    array = new TemplateSignature[0];
                currentOB_.getOd().setSignatures(array);

                if (templateModel_.getCurrColumn() == -1) {
                    removeTemplate_.setText("Delete");
                    duplicateTemplate_.setText("Duplicate");
                    removeTemplate_.setEnabled(false);
                    duplicateTemplate_.setEnabled(false);
                } else {
                    templateTable_.setRowColumn(
                        0,
                        templateModel_.getCurrColumn());
                    templateTable_.requestFocus();
                }
            }
        });

        duplicateTemplate_.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                if (templateModel_.getColumnCount() != 0) {
                    templateTable_.stopCellEditing();
                    int column = templateTable_.getSelectedColumn();
                    Object item = templateModel_.getDataItem(column);
                    TemplateSignature ts = null;
                    if (item instanceof TemplateSignature) {
                        if (!((TemplateSignature) item)
                            .getTemplateType()
                            .equalsIgnoreCase("acquisition")) {
                            ts =
                                new TemplateSignature((TemplateSignature) item);
                            templateModel_.insertColumn(column, ts);
                            currentOB_.getOd().setSignatures(
                                templateModel_.getTemplateSignatures());
                            templateTable_.setRowColumn(
                                0,
                                templateModel_.getCurrColumn());
                            templateTable_.requestFocus();
                        }
                    }
                }
            }

        });

        save_.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                /**
                 * Prompt the user to confirm saving the changes
                 */
                int result =
                    JOptionPane.showConfirmDialog(
                        MainWindow.getPrincipalWindow(),
                        "Are you sure you wish to save the changes?",
                        "Confirm Save?",
                        JOptionPane.YES_NO_OPTION);

                if (result == JOptionPane.YES_OPTION) {

                    Persistence objMgr = Persistence.getInstance();
                    try {
                        objMgr.putBusObj(currentOB_);
                        objMgr.putBusObj(currentOB_.getTarget());
                        objMgr.putBusObj(currentOB_.getOd());
                        objMgr.putBusObj(currentOB_.getConstraintSet());
                        try {
                            OBHistory.addEvent(
                                Config.getCfg().uniqueToTableId(
                                    currentOB_.getId()),
                                null,
                                OHSApplication.getShortDesc(),
                                new Integer(Config.getCfg().getUserName())
                                    .toString(),
                                CalibrationBlock.EDITED_BY_USG);
                        } catch (SQLException e) {
                            ErrorMessages.announceIOError(templateTable_, e);
                        }
                    } catch (ObjectIOException e) {
                        ErrorMessages.announceIOError(templateTable_, e);
                    } catch (ObjectNotFoundException e) {
                        ErrorMessages.announceIOError(templateTable_, e);
                    }
                }
            }
        });

        execTimeButton_.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                if (EvmExecutorImpl.evmConfigured()) {
                    EvmExecutorImpl evm = EvmExecutorImpl.getInstance();
                    //Check that the evm is initialized
                    if (evm.isInitialized()) {
                        try {
                            if (!inst_.hasExecTimeCalc()) {
                                displayNotFoundError();
                                return;
                            }
                            evm.calculateETCS(currentOB_);
                            currentOB_.setExecTimeUpToDate(true);
                            AcquisitionModel model =
                                (AcquisitionModel) templateTable_.getModel();
                            model.fireTableDataChanged();
                        } catch (EVMException ex) {
                            displayExecError();
                        }

                    }
                }

            }
        });
    }

    private void displayExecError() {
        JOptionPane.showMessageDialog(
            this,
            "Exception in Exectime script",
            "Exception in Exectime script",
            JOptionPane.ERROR_MESSAGE);
    }

    private void displayNotFoundError() {
        JOptionPane.showMessageDialog(
            this,
            "Exec Time Calculator not available for this instrument",
            "Execution Time Calculatar not found",
            JOptionPane.ERROR_MESSAGE);
    }

    /**
     * Updates the DocView components to refer to the correct OB.
     *
     * @param newOB     ObservationBlock object to be displayed.
     * Creates a component with subcomponents linked to the
     * ObservationBlock object's fields.  When the returned component is
     * displayed, changes to it will be reflected in the ObservationBlock
     * object.  Som6e fields of an ObservationBlock may not be visible.
     * Some fields may not be modifiable.
     *
     * @param ob the ObservationBlock object to link to.
     *
     */
    public void setObject(ObservationBlock newOB) {
//        if (newOB == currentOB_ && !Config.getCfg().isInEngineeringMode()) {
//            boolean runChanged = false;
//            if(currentOB_ != null && newOB != null) {
//                if( currentOB_.getObsRun().getId() != newOB.getObsRun().getId())
//                    runChanged = true;
//            }
//            System.out.println("Run Changed " + runChanged);
//            if(!runChanged)
//                return;
//        }
        boolean recreateCSGUI=false;
        if (currentOB_ != null) {
            currentOB_.removePropertyChangeListener(this);
            if(currentOB_.getObsRun().getId() != newOB.getObsRun().getId() ||
               !currentOB_.getOd().getInstrument().equals(newOB.getOd().getInstrument())) {
                recreateCSGUI=true;
            }    
        }
        if(currentOB_==null && newOB != null)
            recreateCSGUI=true;
        
        currentOB_ = newOB;
        if (currentOB_ != null) {
            ObservationDescription od = currentOB_.getOd();
            fieldODName_.setObjectProperty(od, "Name");
        }
        
        boolean canModify = isAllowedToModify(currentOB_);
        boolean forceAcqModelReload = false;

        /*
         * MIPS Project:
         * If the Observing Run IP version differs from the IP version in the
         * ObservationDescription of the currentOB_, and the OB is modifyable,
         * then it must be converted to the new IP version.
         */
        float requiredIPVersion=-1;

        if (canModify) {
            ObservingRun obr = currentOB_.getObsRun();
            if (obr != null) {
                boolean singleIPMode =
                    OHSConfig.getInstance().getBoolean(
                        OHSKeys.IPVersioningSingleIPModeEnabled);
                //float requiredIPVersion;
                if (singleIPMode) {
                    // in single IP mode the required IP version
                    // is driven by the Instrument in the InstrumentList
                    // not from the Observing Run
                    Instrument inst =
                        InstrumentList.getInstance().getInstrument(
                            obr.getInstCode(),
                            obr.getIPVersion());
                    requiredIPVersion = inst.getVersionNumber();
                } else {
                    // in multiple IP mode the required IP version
                    // is driven by the Observing Run
                    requiredIPVersion = obr.getIPVersion();
                }
                float obIPVersion = currentOB_.fetchIPVersionFromOd();
                if (requiredIPVersion != obIPVersion) {
                    recreateCSGUI=true;
                    // The OB must be converted
                    stdlog_.debug(
                        "MIPS This OB is v"
                            + obIPVersion
                            + " but must be converted to v"
                            + requiredIPVersion);
                    try {
                        convertOB(requiredIPVersion);
                        try {
                            stdlog_.debug("!!!!!!!! Before Conversion " + currentOB_.getConstraintSet().getBaseline());

                            ConstraintProperties.refreshConstraintSets(currentOB_,requiredIPVersion);
                          stdlog_.debug("!!!!!!!! Current OB BASELINE " + currentOB_.getConstraintSet().getBaseline());
                        } catch (NoSuchMethodException e1) {
                            //TODO Auto-generated catch block
                            e1.printStackTrace();
                        } catch (IllegalAccessException e1) {
                            //TODO Auto-generated catch block
                            e1.printStackTrace();
                        } catch (InvocationTargetException e1) {
                            //TODO Auto-generated catch block
                            e1.printStackTrace();
                        }
                    } catch (IOException e) {
                        stdlog_.debug("MIPS OB CONVERSION FAILED");
                        e.printStackTrace();
                    }
                    if (!singleIPMode
                        && OHSConfig.getInstance().getBoolean(
                            OHSKeys.ShowConversionDialog)) {
                        JOptionPane.showMessageDialog(
                            this,
                            new Object[] {
                                "This Observation Block has been updated to use",
                                "the Instrument Package specified in the Observing Run." },
                            "Update to new Instrument Package",
                            JOptionPane.INFORMATION_MESSAGE);
                    }
                    stdlog_.debug("MIPS OB CONVERSION EXECUTED");
                    forceAcqModelReload = true;
                }
            }
        }
        if(currentOB_ != null) {
            float obIPVersion = currentOB_.fetchIPVersionFromOd();
            String instCode = currentOB_.getOd().getInstrument();
            Instrument inst = InstrumentList.getInstance().getInstrument(instCode,obIPVersion);
            if(inst.hasInstComments()) {
                fieldInstComments_.setEnabled(true);
                instComments_.setEnabled(true);
                instComments_.setText("Instrument Comments : " +inst.getLabelInstComment());
                instComments_.setForeground(Color.BLUE);
                hasInstComments = true;
                //instComments_
            } else {
                instComments_.setText("Instrument Comments : ");
                instComments_.setForeground(Color.BLACK);
                instComments_.setEnabled(false);
                fieldInstComments_.setEnabled(false);
                hasInstComments = false;
            }

            if(AppConfig.getAppConfig().hasSTTime(currentOB_.getOd().getInstrument())) {
                tabPane_.setEnabledAt(3,true);
            } else {
                tabPane_.setEnabledAt(3,false);     
            }            
        }                
        if(currentOB_ != null) {
            if(AppConfig.getAppConfig().hasSTTime(currentOB_.getOd().getInstrument())) {
                tabPane_.setEnabledAt(3,true);
            } else {
                tabPane_.setEnabledAt(3,false);     
            }            
        }        
        fieldName_.setObjectProperty(currentOB_, "Name");

        fieldStatus_.setObjectProperty(currentOB_, "Status");

        fieldUserComments_.setObjectProperty(currentOB_, "UserComments");
        
        fieldInstComments_.setObjectProperty(currentOB_, "InstrumentComments");


        fieldCalibrationRequirements_.setObjectProperty(
            currentOB_,
            "CalibrationRequirements");

        fieldUserPriority_.setObjectProperty(currentOB_, "UserPriority");

        /* cmavros added 30/Aug/2001 */
        fieldExecutionTime_.setObjectProperty(currentOB_, "ExecutionTime");

        if (currentOB_ != null)
            initLists();

        tiView_.setObject(currentOB_);
        stView_.setObject(currentOB_);
        if (currentOB_ != null) {
            currentOB_.addPropertyChangeListener(this);
            csView_.setObject(currentOB_.getConstraintSet(),recreateCSGUI,currentOB_.getOd().getInstrument(),currentOB_.getOd().getIPVersion()); 
            //if(recreateCSGUI) {
                
                    if(requiredIPVersion!=-1) {
                        if(tabPane_.getSelectedIndex()==1) {
                            // complete hack 
                            //TODO only way to refresh panel I could find
                            tabPane_.setSelectedIndex(0);
                            tabPane_.setSelectedIndex(1);
                        }
                            
                    }
            recreateCSGUI=false;
            targetView_.setObject(currentOB_.getTarget());
            templateModel_ =
                AcquisitionModel.getAcqModel(currentOB_, forceAcqModelReload);
            int columnCount = templateModel_.getColumnCount();
            templateTable_.setModel(templateModel_);
            if (templateModel_.getColumnCount() > 0) {
                templateTable_.setRowColumn(
                    0,
                    templateModel_.getColumnCount() - 1);
            } else {
                removeTemplate_.setText("Delete");
                duplicateTemplate_.setText("Duplicate");
                removeTemplate_.setEnabled(false);
                duplicateTemplate_.setEnabled(false);
            }
            stdlog_.debug("!!!!!!!!!!!!!!! In set Object");
            if (!currentOB_.isExecTimeUpToDate()
                || currentOB_.getExecutionTime() <= 0) {
                execTimeLabel_.setText("* Execution Time");
            } else
                execTimeLabel_.setText("Execution Time");
        }
        fieldName_.requestFocus();
        addTemplate_.setEnabled(false);
        setEnabled(canModify);
        if (Config.getCfg().isInOperationsMode()) {
            putInEditMode();
        } else {
            save_.setVisible(false);
            save_.setEnabled(false);
        }
    }

    /**
     * MIPS Project:
     * Convert the currentOB_ to the specified ip version
     * (instrument stays the same, of course!).
     * Method assumes that currentOB_ is non-null.
     * @throws IOException if the conversion (template refresh) failed.
     */
    private void convertOB(float obrIPVersion) throws IOException {
        currentOB_.convertIPVersion(obrIPVersion);
        // refresh templates (forcing a full refresh) and fire the autosaver.
        currentOB_.refreshTemplates(true, true);
    }

    private class OBTemplateTable extends ODTemplateTable {

        /**
         * Create a reference to a JTable
         * @param model initialise this table with this model
         */
        OBTemplateTable(TableModel model) {
            super(model);
        }

        /**
         * Create a table using the default model
         */
        OBTemplateTable() {
            super();
        }

        public void setModel(TableModel model) {
            super.setModel(model);
        }

        public void columnSelectionChanged(ListSelectionEvent ev) {
            super.columnSelectionChanged(ev);
            if (ev.getValueIsAdjusting())
                return;
            //             stdlog_.debug("CLASS OBView$OBTemplateTable - columnSelectionChanged: " +
            //                 "List Selection event " + ev);
            boolean canModify = isAllowedToModify(currentOB_);
            int column = templateTable_.getSelectedColumn();
            int columnCount = templateModel_.getColumnCount();

            if (Config.getCfg().isInOperationsMode()) {
                return;
            }
            if (columnCount == 0 || !canModify) {
                removeTemplate_.setText("Delete");
                duplicateTemplate_.setText("Duplicate");
                removeTemplate_.setEnabled(false);
                duplicateTemplate_.setEnabled(false);
                if (!canModify)
                    addTemplate_.setEnabled(false);
                return;
            }
            // Java1.4 port if(column >= columnCount)
            if (column >= columnCount || column < 0)
                return;
            removeTemplate_.setEnabled(true);
            duplicateTemplate_.setEnabled(true);
            if (currentOB_.getTemplateSignature() != null) {
                if (columnCount <= 2)
                    duplicateTemplate_.setEnabled(false);
                if (column <= 1)
                    duplicateTemplate_.setEnabled(false);
            }
            if (templateModel_.isLabel(column)) {
                duplicateTemplate_.setEnabled(false);
                column++;
            }
            removeTemplate_.setText("Delete Col : " + (column + 1));
            duplicateTemplate_.setText("Duplicate Col : " + (column + 1));
        }
    }
}
