package org.eso.ohs.phase2.apps.p2pp.views;

import java.awt.Color;
import java.awt.Component;
import java.awt.Rectangle;

import javax.swing.BorderFactory;
import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JTable;
import javax.swing.table.TableCellRenderer;

import org.eso.ohs.core.gui.widgets.MultiLineToolTip;
import org.eso.ohs.core.gui.widgets.TableRendererPerformance;
import org.eso.ohs.instruments.Parameter;
import org.eso.ohs.instruments.TemplateSignature;

/**
 * This class reneders the value for every cell on the grid.
 * It also sets the tooltip text for the 
 * 
 * <p>
 * $Id: RendererFactory.java,v 1.11 2005/11/16 00:55:40 tcanavan Exp $
 * @author Tim Canavan
 */

public class RendererFactory implements TableCellRenderer,TableRendererPerformance {

    /** 
     * Current parameter being rendered
     */

    private Parameter currParam_;

    /**
     * Component to be displayed in the grid.
     */
    private JLabel rendererComponent;

    /**
     * Render this checkbox if the currParam_ is a boolean value
     */

    private JCheckBox rendererComponentCheckBox;

    /**
     * Tooltp to be displayed when the mouse is over a 
     * particular cell.
     */

    private String tooltp_;

    /** Reference to a multiline tooltip control */
    private static MultiLineToolTip multiLineToolTip_ = new MultiLineToolTip();

    /**
     * Default constructor
     */

    public RendererFactory() {
        rendererComponent = new JLabel("");
        rendererComponentCheckBox = new JCheckBox(null, null, false);
        rendererComponentCheckBox.setBorderPainted(true);
    }

    /**
     * Return the appropiate component for a given cell and set its tool-tip text 
     * 
     * @param table JTable reference
     * @param value value of cell
     * @param isSelected selected value
     * @param hasFocus focus
     * @param row row index
     * @param column column index    
     */

    public Component getTableCellRendererComponent(
        JTable table,
        Object value,
        boolean isSelected,
        boolean hasFocus,
        int row,
        int column) {
        //Component component = super.getTableCellRendererComponent(table,value,isSelected,h//asFocus,
        //                                row,column);                    

        boolean range = false;
        boolean validItems = false;
        boolean renderTemplateSignature = false;
        TemplateModel model = (TemplateModel) table.getModel();
        Object obj = model.getDataItem(column);
        if (obj instanceof TemplateSignature && row != -1) {
            TemplateSignature ts = (TemplateSignature) obj;
            renderTemplateSignature = true;
            Parameter[] plist = ts.getDisplayParamList();
            int rowCount = plist.length;
            if (plist != null) {
                if (row < rowCount) {
                    currParam_ = plist[row];
                    tooltp_ = "";
                    if (currParam_.isFixed() == true)
                        tooltp_ = "Fixed Value cannot be edited";
                    else
                        tooltp_ = currParam_.getToolTipText();
                    String miniHelp = currParam_.getMiniHelp();
                    if (miniHelp != null && miniHelp.length() > 0) {
                        if (currParam_.getType().equals("paramfile")
                            || currParam_.getType().equals("file"))
                            tooltp_ = tooltp_ + "\n " + "Help : " + miniHelp;
                        else
                            tooltp_ =
                                "Range : "
                                    + tooltp_
                                    + "\n "
                                    + "Help : "
                                    + miniHelp;
                    }
                    if (currParam_.getType().equals("boolean")) {
                        if ("True".equalsIgnoreCase(currParam_.getValue())
                            || "T".equalsIgnoreCase(currParam_.getValue())) {
                            rendererComponentCheckBox.setSelected(true);
                            //rendererComponentCheckBox.setText("True");
                        } else {
                            rendererComponentCheckBox.setSelected(false);
                            //rendererComponentCheckBox.setText("False");
                        }
                        if (isSelected)
                            rendererComponentCheckBox.setBorder(
                                BorderFactory.createLineBorder(
                                    Color.yellow,
                                    1));
                        else
                            rendererComponentCheckBox.setBorder(
                                BorderFactory.createLineBorder(Color.black, 0));
                        rendererComponentCheckBox.setToolTipText(tooltp_);
                        return rendererComponentCheckBox;
                    } else {               
                        rendererComponent.setToolTipText(tooltp_);
                        rendererComponent.setText((String) value);
                        if (!currParam_.verifyValue((String) value)) {
                            rendererComponent.setForeground(Color.red);

                            //org.eso.ohs.utilities.DebugUtils.stackTrace();
                        } else
                            rendererComponent.setForeground(Color.black);
                    }

                } else {
                    rendererComponent.setText("");
                }

            }
        } else {
            rendererComponent.setText((String) value);
            rendererComponent.setForeground(Color.black);
            rendererComponent.setToolTipText((String) value);
        }
        if (isSelected)
            rendererComponent.setBorder(
                BorderFactory.createLineBorder(Color.yellow, 1));
        else
            rendererComponent.setBorder(
                BorderFactory.createLineBorder(Color.black, 0));
        return rendererComponent;
    }
    
    // The below methods are deliberately left empty for
    // performance reasons.

	public void validate() {
		
	}

	public void revalidate() {
		
	}

	public void repaint(long arg, int arg1, int arg2, int arg3) {
		
	}

	public void repaint(Rectangle rectangle) {
		
	}    

}
