package org.eso.ohs.phase2.visiplot;

/*
 * MiscOptionsPanel.java
 *
 * Created on July 31, 2001, 2:59 PM
 *
 * @author Nick Kornweibel for ESO.
 */

import java.awt.BorderLayout;
import java.awt.GridLayout;

import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextArea;
import javax.swing.SwingConstants;

public class MiscOptionsPanel extends JPanel {

    private JPanel northPanel_;
    private JLabel precisionLabel_;
    private JComboBox precisionCombo_;
    private JLabel printerLabel_;
    private JTextArea printerEdit_;

    private static int precision[] = {10, 30, 60};
	private static int defaultPrecisionIndex = 0; //i.e. the value 10. 

    /**
     * Creates new form MiscOptionsPanel
     */

    public MiscOptionsPanel() {
        initComponents();
    }

    /**
     * This method is called from within the constructor to
     * initialize the form.
     */

    private void initComponents() {
        northPanel_ = new JPanel(new GridLayout(0, 2, 5, 5));
        precisionLabel_ = new JLabel();
        precisionCombo_ = new JComboBox();
        printerLabel_ = new JLabel();
        printerEdit_ = new JTextArea();
        setLayout(new BorderLayout());

        precisionLabel_.setText("Plot precision:");
        printerLabel_.setText("Printer:");
        precisionLabel_.setHorizontalAlignment(SwingConstants.RIGHT);
        printerLabel_.setHorizontalAlignment(SwingConstants.RIGHT);
        northPanel_.add(precisionLabel_);
        northPanel_.add(precisionCombo_);
        northPanel_.add(printerLabel_);
        northPanel_.add(printerEdit_);
        add(northPanel_, BorderLayout.NORTH);

        for (int i = 0; i < precision.length; i++) {
            precisionCombo_.addItem(getLabel(precision[i]));
        }
    }

    private static String getLabel(int prec) {
        return "1 per "+prec+" minutes";
    }

    /**
     * Set the combo selection to the specified precision - if not
     * an item in the combo, the selection fails.
     */

    public void setPrecision(int prec) {
        precisionCombo_.setSelectedItem(getLabel(prec));
    }

    /**
     * Get the current precision setting. Returns -1 if the
     * precision could not be determined.
     */

    public int getPrecision() {
        int rc = -1;
        String precStr = (String)precisionCombo_.getSelectedItem();
        for  (int i = 0; i < precision.length; i++) {
            if (getLabel(precision[i]).equals(precStr)) {
                rc = precision[i];
                break;
            }
        }
        return rc;
    }

    /**
     * Set the printer name
     */

    public void setPrinterName(String name) {
        printerEdit_.setText(name);
    }

    /**
     * @return the printer name entered
     */

    public String getPrinterName() {
        return printerEdit_.getText();
    }

    /**
     * Check all entries in the panel. 
     * Precision combo is set to 30 if it is empty.
     * @return a report on the validity of the entries in the panel, ""
     * indicates that there is no problem with the entries.
     */	

    public String checkEntries() {
        if (getPrecision() == -1) {
            setPrecision(defaultPrecisionIndex);
        }
        return "";
    }
}
