package org.eso.ohs.scripting;

/** 
 * An adaptor class from TargetAPI to Target
 * This class accepts a Target and exposes the TargetAPI.
 *
 * @author Nick Kornweibel
 */

import org.eso.ohs.dfs.Target;
import org.eso.ohs.dfs.docviewdatatrans.DecAdaptor;
import org.eso.ohs.dfs.docviewdatatrans.RaAdaptor;
import org.eso.ohs.persistence.Config;

public class TargetAdaptor implements TargetAPI {

    private Target target_;

    /**
     * Constructor accepts a Target object, assuming non-null
     * @param target object
     */

    public TargetAdaptor(Target targ) {
        target_ = targ;
    }

    /**
     * Get the name of the target for this OB
     * @return string containing the name of the target
     */

    public String getName() {
        return ScriptingUtilities.nullToEmptyString(target_.getName());
    }

    /**
     * Get the Id of this target
     * @return string id for this target (long value in String)
     */

    public String getId() {
        String rc;
        if (target_.getDbaseId() != 0) {
            rc = ""+Config.getCfg().uniqueToTableId(target_.getDbaseId());
        } else {
            rc = ""+Config.getCfg().uniqueToTableId(target_.getId());
        }
        return rc;
    }


    /**
     * Get the string form of the RA value for this OB
     * @return string containing the RA e.g. "05:45:36.5"
     */

    public String getRightAscension() {
        RaAdaptor raAdap = new RaAdaptor();
        return raAdap.fromMetaLevelObject(new Integer(target_.getRA()));
    }

    /**
     * Get the string form of the DEC value for this OB
     * @return string containing the DEC e.g. "-78:11:09.0"
     */

    public String getDeclination() {
        DecAdaptor decAdap = new DecAdaptor();
        return decAdap.fromMetaLevelObject(
                new Integer(target_.getDeclination()));
    }

    /**
     * Get the Equinox value for this target
     * @return String containing the equinox for this target e.g. "J2000"
     */

    public String getEquinox() {
        return ScriptingUtilities.nullToEmptyString(target_.getEquinox());
    }

    /**
     * Get the Epoch value for this target
     * @return string value for the epoch for this target e.g. "2000.0"
     */

    public String getEpoch() {
        return ""+target_.getEpoch();
    }
}
